/***************************************************************
 * Name:      uhradydialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "uhradydialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"

void UhradyCtrl::CreateColumns()
{
    InsertColumn(UCOL_DATUM, "Datum");
    InsertColumn(UCOL_CISLO, "Doklad");
    InsertColumn(UCOL_POPIS, "Popis");
    SetColumnWidth(2, 250);
    InsertColumn(UCOL_CASTKA, wxString::FromUTF8("Částka"), wxLIST_FORMAT_RIGHT);
    setResizeColumn(UCOL_POPIS);
}

wxString UhradyCtrl::OnGetItemText(long item, long column) const
{
    return m_dialog->getItemText(item, column);
}

int UhradyCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *UhradyCtrl::OnGetItemAttr(long /*item*/) const
{
    return nullptr;
}

IMPLEMENT_DYNAMIC_CLASS(UhradyDialog, wxDialog)

BEGIN_EVENT_TABLE(UhradyDialog, wxDialog)
    EVT_BUTTON(wxID_OK, UhradyDialog::OnOk)
    EVT_LIST_COL_CLICK(ID_UHRADY_LIST, UhradyDialog::OnColumnClick)
END_EVENT_TABLE()

UhradyDialog::UhradyDialog(wxWindow *parent, int fakturaid, int zavpohid)
    : wxDialog(parent, wxID_ANY, wxEmptyString), m_fakturaid(fakturaid), m_zavpohid(zavpohid), m_sortCol(UhradyCtrl::UCOL_DEF_SORT), m_sortAsc(true)
{
    if(idbmodel->hasFaktura(m_fakturaid))
    {
        Faktura fa = idbmodel->getFaktura(m_fakturaid);
        SetTitle(wxString::Format(wxString::FromUTF8("Úhrady faktury %s"), fa.cislo()));
        m_denik = idbmodel->denikUhradForFaktura(fa.id());
    }
    else
    {
        ZavPoh zp = idbmodel->getZavPoh(m_zavpohid);
        if(zp.isValid())
        {
            if(zp.zavazek()) SetTitle(wxString::Format(wxString::FromUTF8("Úhrady závazku %s"), zp.cislo()));
            else SetTitle(wxString::Format(wxString::FromUTF8("Úhrady pohledávky %s"), zp.cislo()));
            m_denik = idbmodel->denikUhradForZavpoh(zp.id());
        }
    }
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

UhradyDialog::~UhradyDialog()
{
    m_denik.Clear();
}

wxString UhradyDialog::getItemText(long item, long column) const
{
    if(item < 0 || item >= (long)m_denik.GetCount())
        return "";
    switch(column)
    {
    case UhradyCtrl::UCOL_DATUM:
        return m_denik[item].datum().FormatDate();
    case UhradyCtrl::UCOL_CISLO:
        return m_denik[item].cislo();
    case UhradyCtrl::UCOL_POPIS:
        return m_denik[item].popis();
    case UhradyCtrl::UCOL_CASTKA:
        return wxString::Format("%.2f", m_denik[item].castka());
    default:
        return "";
    }
}

void UhradyDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new UhradyCtrl(this, this, ID_UHRADY_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    updateData();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void UhradyDialog::updateData()
{
    size_t oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    idbmodel->sortDenik(m_denik, columnToDenikSort(m_sortCol), m_sortAsc);
    size_t cnt = m_denik.GetCount();
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}

void UhradyDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void UhradyDialog::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= UhradyCtrl::UCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<UhradyCtrl::UColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    updateData();
}

DbModel::DSort UhradyDialog::columnToDenikSort(UhradyCtrl::UColumn column)
{
    switch(column)
    {
    case UhradyCtrl::UCOL_CASTKA: return DbModel::DS_CASTKA;
    case UhradyCtrl::UCOL_CISLO: return DbModel::DS_CISLO;
    case UhradyCtrl::UCOL_DATUM: return DbModel::DS_DATUM;
    case UhradyCtrl::UCOL_POPIS: return DbModel::DS_POPIS;
    default: return DbModel::DS_DATUM;
    }
}

void UhradyDialog::setColumnImage(UhradyCtrl::UColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}
