/***************************************************************
 * Name:      filter.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include "filter.h"
#include "utils/dxutils.h"
#include <wx/arrimpl.cpp>

WX_DEFINE_OBJARRAY(dxFilterArray)

Filter::Filter(const FilterAction &action, bool active, const wxDateTime &datum) : m_type(FT_DATUM), m_action(action), m_active(active), m_datum(datum), m_intvalue(0), m_doublevalue(0.0)
{
}

Filter::Filter(const FilterAction &action, bool active, const wxString &text) : m_type(FT_TEXT), m_action(action), m_active(active), m_text(text), m_intvalue(0), m_doublevalue(0.0)
{
  }

Filter::Filter(const FilterAction &action, bool active,  int value) : m_type(FT_INTEGER), m_action(action), m_active(active), m_intvalue(value), m_doublevalue(0.0)
{
}

Filter::Filter(const FilterAction &action, bool active, double value) : m_type(FT_DOUBLE), m_action(action), m_active(active), m_intvalue(0), m_doublevalue(value)
{
}

bool Filter::matchFilter(const wxDateTime &datum)
{
    if(!m_active) return true;
    switch (m_action) {
    case FA_LESS:
        return datum.IsEarlierThan(m_datum);
    case FA_LESSEQUAL:
        return datum.IsEarlierThan(m_datum)||datum.IsEqualTo(m_datum);
    case FA_EQUAL:
        return datum.IsEqualTo(m_datum);
    case FA_GREATEREQUAL:
        return datum.IsLaterThan(m_datum)||datum.IsEqualTo(m_datum);
    case FA_GREATER:
        return datum.IsLaterThan(m_datum);
    default:
        break;
    }
    return false;
}

bool Filter::matchFilter(const wxString &text)
{
    if(!m_active) return true;
    switch (m_action) {
    case FA_EQUAL:
        return text.Lower() == m_text.Lower();
    case FA_CONTAINS:
        return text.Lower().Contains(m_text.Lower());
    default:
        break;
    }
    return false;
}

bool Filter::matchFilter(int value)
{
    if(!m_active) return true;
    switch (m_action) {
    case FA_LESS:
        return value < m_intvalue;
    case FA_LESSEQUAL:
        return value <= m_intvalue;
    case FA_EQUAL:
        return value == m_intvalue;
    case FA_GREATEREQUAL:
        return value >= m_intvalue;
    case FA_GREATER:
        return value > m_intvalue;
    default:
        break;
    }
    return false;
}

bool Filter::matchFilter(double value)
{
    if(!m_active) return true;
    switch (m_action) {
    case FA_LESS:
        return value < m_doublevalue;
    case FA_LESSEQUAL:
        return value <= m_doublevalue;
    case FA_EQUAL:
        return dxutils::isSameDouble(value,m_doublevalue);
    case FA_GREATEREQUAL:
        return value >= m_doublevalue;
    case FA_GREATER:
        return value > m_doublevalue;
    default:
        break;
    }
    return false;
}

FilterType Filter::type() const
{
    return m_type;
}

void Filter::setType(const FilterType &type)
{
    m_type = type;
}

FilterAction Filter::action() const
{
    return m_action;
}

void Filter::setAction(const FilterAction &action)
{
    m_action = action;
}

wxDateTime Filter::datum() const
{
    return m_datum;
}

void Filter::setDatum(const wxDateTime &datum)
{
    m_datum = datum;
}

wxString Filter::text() const
{
    return m_text;
}

void Filter::setText(const wxString &text)
{
    m_text = text;
}

int Filter::intvalue() const
{
    return m_intvalue;
}

void Filter::setIntvalue(int intvalue)
{
    m_intvalue = intvalue;
}

double Filter::doublevalue() const
{
    return m_doublevalue;
}

void Filter::setDoublevalue(double doublevalue)
{
    m_doublevalue = doublevalue;
}

bool Filter::active() const
{
    return m_active;
}

void Filter::setActive(bool active)
{
    m_active = active;
}

