/***************************************************************
 * Name:      dphdp3print.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/stdpaths.h>
#include <wx/filename.h>
#include "dphdp3print.h"
#include "utils/dxutils.h"
#include "wx/pdfdc.h"
#include "wx/pdffontmanager.h"
#include "model/dbmodel.h"
#include "config.h"

const wxString specialTags[] = {"{c_okec}", "{c_pop}", "{c_ufo}", "{c_pracufo}", "{typ_platce}", "{trans}", "{dapdph_forma}"};
const size_t numSpecialTags = (sizeof(specialTags)/sizeof(specialTags[0]));

bool isSpecialTag(const wxString &line)
{
    for(size_t i=0; i<numSpecialTags; i++)
    {
        if(line.Contains(specialTags[i]))
            return true;
    }
    return false;
}

Dphdp3Print::Dphdp3Print(wxWindow *parent, dxAtributArray atributes)
    : DocumentPrint(parent, wxString::FromUTF8("Přiznání k DPH")), m_atributes(atributes)
{
    m_document = new Document();
    parseXml();
    int posY = m_document->marginTop()-PAGE_OFFSET;
    int sheight = m_document->sectionHeight(HEADER1);
    moveElements(m_document->elementsBySection(HEADER1), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(HEADER2);
    moveElements(m_document->elementsBySection(HEADER2), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(HEADER3);
    moveElements(m_document->elementsBySection(HEADER3), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(ITEMS);
    moveElements(m_document->elementsBySection(ITEMS), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(SUM);
    moveElements(m_document->elementsBySection(SUM), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(DPH);
    moveElements(m_document->elementsBySection(DPH), posY, sheight);
    sheight = m_document->sectionHeight(FOOTER1);
    posY = 297-m_document->marginBottom()-sheight;
    moveElements(m_document->elementsBySection(FOOTER1), posY, sheight);
    sheight = m_document->sectionHeight(FOOTER2);
    posY -= sheight;
    moveElements(m_document->elementsBySection(FOOTER2), posY, sheight);
    sheight = m_document->sectionHeight(FOOTER3);
    posY -= sheight;
    moveElements(m_document->elementsBySection(FOOTER3), posY, sheight);
    moveElements(m_document->elementsBySection(ANYSECTION), 0, 0);
}

void Dphdp3Print::Preview(bool print)
{
    Dphdp3Printout *print1 = new Dphdp3Printout(printTitle());
    print1->setDocument(m_document);
    print1->setHeaderLeftText(m_headerLeftText);
    print1->setHeaderCenterText(m_headerCenterText);
    print1->setHeaderRightText(m_headerRightText);
    print1->setHeaderLine(m_headerLine);
    print1->setMarginLeft(m_marginLeft);
    print1->setMarginRight(m_marginRight);
    print1->setMarginTop(m_marginTop);
    print1->setMarginBottom(m_marginBottom);
    print1->setPages(m_document->pages());
    print1->performPageSetup();
    wxPrintDialogData data(print1->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    data.SetFromPage(1);
    data.SetToPage(m_document->pages());
    wxPrintPreview *preview = nullptr;
    if(print)
    {
        Dphdp3Printout *print2 = new Dphdp3Printout(printTitle());
        print2->setDocument(m_document);
        print2->setHeaderLeftText(m_headerLeftText);
        print2->setHeaderCenterText(m_headerCenterText);
        print2->setHeaderRightText(m_headerRightText);
        print2->setHeaderLine(m_headerLine);
        print2->setMarginLeft(m_marginLeft);
        print2->setMarginRight(m_marginRight);
        print2->setMarginTop(m_marginTop);
        print2->setMarginBottom(m_marginBottom);
        print2->setPages(m_document->pages());
        print2->performPageSetup();
        preview = new wxPrintPreview(print1, print2, &data);
    }
    else
    {
        preview = new wxPrintPreview(print1, nullptr, &data);
    }
    if (!preview->IsOk())
    {
        delete preview;
        wxMessageBox("There was a problem with previewing.\nPerhaps your current printer is not set correctly?", "Previewing", wxOK);
        return;
    }
    print1->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    preview->SetZoom(PREVIEW_ZOOM);
    wxPreviewFrame *frame = new wxPreviewFrame(preview, m_parent, m_title, wxDefaultPosition, wxSize(PREVIEW_WIDTH,PREVIEW_HEIGHT));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show();
}

void Dphdp3Print::Print()
{
    Dphdp3Printout *print = new Dphdp3Printout(printTitle());
    print->setDocument(m_document);
    print->setHeaderLeftText(m_headerLeftText);
    print->setHeaderCenterText(m_headerCenterText);
    print->setHeaderRightText(m_headerRightText);
    print->setHeaderLine(m_headerLine);
    print->setMarginLeft(m_marginLeft);
    print->setMarginRight(m_marginRight);
    print->setMarginTop(m_marginTop);
    print->setMarginBottom(m_marginBottom);
    print->setPages(m_document->pages());
    print->performPageSetup();
    wxPrintDialogData data(print->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    print->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    wxPrinter printer(&data);
    printer.Print(m_parent, print, true);
}

wxString Dphdp3Print::CreatePDF(const wxString &filename, const wxString &name, bool select)
{
    registerFontsForPdf();
    wxFileName fileName;
    if(select)
    {
        wxFileDialog dialog(m_parent, wxString::FromUTF8("Uložit přiznání k DPH jako"), wxGetHomeDir(), filename+wxT(".pdf"), wxT("PDF soubor (*.pdf)|*pdf"), wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
        if(dialog.ShowModal() == wxID_OK)
        {
            fileName.SetPath(dialog.GetDirectory());
            fileName.SetFullName(dialog.GetFilename());
        }
        else
        {
            return wxEmptyString;
        }
    }
    else
    {
        fileName.SetPath(wxStandardPaths::Get().GetUserDataDir());
        fileName.SetFullName(filename+wxT(".pdf"));
    }
    Dphdp3Printout *print = new Dphdp3Printout(printTitle());
    print->setDocument(m_document);
    print->setHeaderLeftText(m_headerLeftText);
    print->setHeaderCenterText(m_headerCenterText);
    print->setHeaderRightText(m_headerRightText);
    print->setHeaderLine(m_headerLine);
    print->setMarginLeft(m_marginLeft);
    print->setMarginRight(m_marginRight);
    print->setMarginTop(m_marginTop);
    print->setMarginBottom(m_marginBottom);
    print->setPages(m_document->pages());
    print->performPageSetup();
    wxPrintData data = print->printData();
    data.SetFilename(fileName.GetFullPath());
    wxPdfDC dc(data);
    dc.SetResolution(600);
    print->SetDC(&dc);
    print->OnPreparePrinting();
    print->OnBeginPrinting();
    int minPageNum = 1;
    int maxPageNum = m_document->pages();
    if(print->OnBeginDocument(minPageNum, maxPageNum))
    {
        dc.StartDoc("Start PDF...");
        wxPdfDocument *doc = dc.GetPdfDocument();
        if(!name.IsEmpty()) doc->SetTitle(name);
        doc->SetAuthor(idbmodel->firma().nazev());
        doc->SetCreator(wxString(FULLNAME));
        for(int pn = minPageNum; pn <= maxPageNum && print->HasPage(pn); pn++)
        {
            dc.StartPage();
            bool cont = print->OnPrintPage(pn);
            dc.EndPage();
            if(!cont) break;
        }
        print->OnEndDocument();
    }
    print->OnEndPrinting();
    return fileName.GetFullPath();
}

void Dphdp3Print::parseXml()
{
    Document doc;
#if defined (__WXMAC__)
    if(wxDateTime::Today().IsEarlierThan(wxDateTime(1,wxDateTime::Apr,2019))) doc.readFromXml(wxStandardPaths::Get().GetResourcesDir()+"/templates/DPHDP3.xml");
    else if(wxDateTime::Today().IsEarlierThan(wxDateTime(1,wxDateTime::Dec,2020))) doc.readFromXml(wxStandardPaths::Get().GetResourcesDir()+"/templates/DPHDP3-21.xml");
    else if(wxDateTime::Today().IsEarlierThan(wxDateTime(1,wxDateTime::Oct,2021))) doc.readFromXml(wxStandardPaths::Get().GetResourcesDir()+"/templates/DPHDP3-22.xml");
    else doc.readFromXml(wxStandardPaths::Get().GetResourcesDir()+"/templates/DPHDP3-23.xml");
#else
    if(wxDateTime::Today().IsEarlierThan(wxDateTime(1,wxDateTime::Apr,2019))) doc.readFromXml(wxString(DXDE_DATADIR)+"/templates/DPHDP3.xml");
    else if(wxDateTime::Today().IsEarlierThan(wxDateTime(1,wxDateTime::Dec,2020))) doc.readFromXml(wxString(DXDE_DATADIR)+"/templates/DPHDP3-21.xml");
    else if(wxDateTime::Today().IsEarlierThan(wxDateTime(1,wxDateTime::Oct,2021))) doc.readFromXml(wxString(DXDE_DATADIR)+"/templates/DPHDP3-22.xml");
    else doc.readFromXml(wxString(DXDE_DATADIR)+"/templates/DPHDP3-23.xml");
#endif
    m_document->setFromDoc(doc);
    dxElementsArray elements = doc.elements();
    for(size_t i=0; i<elements.GetCount(); i++)
    {
        switch(elements[i]->elementType()) {
        case LINE:
        {
            m_document->addElement(*static_cast<LineElement*>(elements[i]));
            break;
        }
        case RECTANGLE:
        {
            m_document->addElement(*static_cast<RectangleElement*>(elements[i]));
            break;
        }
        case IMAGE:
        {
            m_document->addElement(*static_cast<ImageElement*>(elements[i]));
            break;
        }
        case TEXT:
        {
            TextElement el = *static_cast<TextElement*>(elements[i]);
            if(isSpecialTag(el.value()))
            {
                parseSpecialTag(el);
            }
            else
            {
                el.setValue(parseLine(el.value()));
                m_document->addElement(el);
            }
            break;
        }
        default: ;
        }
    }
}

wxString Dphdp3Print::parseLine(wxString line)
{
    for(size_t i=0; i<m_atributes.GetCount(); i++)
    {
        if(line.Contains(wxT('{')+m_atributes[i].name()+wxT('}')))
        {
            line.Replace(wxT('{')+m_atributes[i].name()+wxT('}'), m_atributes[i].value());
        }
    }
    return line;
}

void Dphdp3Print::moveElements(dxElementsArray elements, int posY, int sectionHeight)
{
    for(size_t i=0; i<elements.GetCount(); i++)
    {
        switch(elements[i]->elementType()) {
        case LINE:
        {
            LineElement *el = static_cast<LineElement*>(elements[i]);
            if(el->anchor())
            {
                el->setStart(wxPoint(el->start().x, el->start().y+posY));
                el->setEnd(wxPoint(el->end().x, el->end().y+posY+sectionHeight));
                break;
            }
            if(el->borderTop())
            {
                el->setStart(wxPoint(el->start().x, posY));
                el->setEnd(wxPoint(el->end().x, posY));
                break;
            }
            if(el->borderBottom())
            {
                el->setStart(wxPoint(el->start().x, el->start().y+posY+sectionHeight));
                el->setEnd(wxPoint(el->end().x, el->end().y+posY+sectionHeight));
                break;
            }
            el->setStart(wxPoint(el->start().x, el->start().y+posY));
            el->setEnd(wxPoint(el->end().x, el->end().y+posY));
            break;
        }
        case RECTANGLE:
        {
            RectangleElement *el = static_cast<RectangleElement*>(elements[i]);
            if(el->anchor())
            {
                el->setStart(wxPoint(el->start().x, el->start().y+posY));
                el->setSize(wxSize(el->size().x, el->size().y+sectionHeight));
                break;
            }
            el->setStart(wxPoint(el->start().x, el->start().y+posY));
            break;
        }
        case IMAGE:
        {
            ImageElement *el = static_cast<ImageElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        case TEXT:
        {
            TextElement *el = static_cast<TextElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        case TEXTPARAGRAPH:
        {
            TextParagraphElement *el = static_cast<TextParagraphElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        case TABLEROW:
        {
            TableRowElement *el = static_cast<TableRowElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        default: ;
        }
    }
}

void Dphdp3Print::parseSpecialTag(TextElement el)
{
    if(el.value().Contains("{c_okec}"))
    {
        wxString c_nace = "";
        for(size_t i=0; i<m_atributes.GetCount(); i++)
        {
            if(m_atributes[i].name() == "c_okec")
            {
                c_nace = m_atributes[i].value();
                break;
            }
        }
        wxXmlDocument okec;
        okec.Load(wxStandardPaths::Get().GetUserDataDir()+"/okec.xml");
        if(okec.IsOk() && okec.GetRoot()->GetName() == "Ciselnik")
        {
            wxXmlNode *child = okec.GetRoot()->GetChildren();
            while(child)
            {
                if(c_nace == child->GetAttribute("c_nace"))
                {
                    el.setValue(child->GetAttribute("naz_nace"));
                    m_document->addElement(el);
                    break;
                }
                child = child->GetNext();
            }
        }
        return;
    }
    if(el.value().Contains("{c_pop}"))
    {
        wxString c_orient = "";
        wxString c_pop = "";
        for(size_t i=0; i<m_atributes.GetCount(); i++)
        {
            if(m_atributes[i].name() == "c_pop")
            {
                c_pop = m_atributes[i].value();
            }
            if(m_atributes[i].name() == "c_orient")
            {
                c_orient = m_atributes[i].value();
            }
        }
        el.setValue(c_orient.IsEmpty()?c_pop:c_pop+"/"+c_orient);
        m_document->addElement(el);
        return;
    }
    if(el.value().Contains("{c_ufo}"))
    {
        wxString c_ufo = "";
        for(size_t i=0; i<m_atributes.GetCount(); i++)
        {
            if(m_atributes[i].name() == "c_ufo")
            {
                c_ufo = m_atributes[i].value();
                break;
            }
        }
        wxXmlDocument okec;
        okec.Load(wxStandardPaths::Get().GetUserDataDir()+"/ufo.xml");
        if(okec.IsOk() && okec.GetRoot()->GetName() == "Ciselnik")
        {
            wxXmlNode *child = okec.GetRoot()->GetChildren();
            while(child)
            {
                if(c_ufo == child->GetAttribute("c_ufo"))
                {
                    el.setValue(child->GetAttribute("nazu_ufo"));
                    m_document->addElement(el);
                    break;
                }
                child = child->GetNext();
            }
        }
        return;
    }
    if(el.value().Contains("{c_pracufo}"))
    {
        wxString c_pracufo = "";
        for(size_t i=0; i<m_atributes.GetCount(); i++)
        {
            if(m_atributes[i].name() == "c_pracufo")
            {
                c_pracufo = m_atributes[i].value();
                break;
            }
        }
        wxXmlDocument okec;
        okec.Load(wxStandardPaths::Get().GetUserDataDir()+"/pracufo.xml");
        if(okec.IsOk() && okec.GetRoot()->GetName() == "Ciselnik")
        {
            wxXmlNode *child = okec.GetRoot()->GetChildren();
            while(child)
            {
                if(c_pracufo == child->GetAttribute("c_pracufo"))
                {
                    el.setValue(child->GetAttribute("nazu_pracufo"));
                    m_document->addElement(el);
                    break;
                }
                child = child->GetNext();
            }
        }
        return;
    }
    if(el.value().Contains("{typ_platce"))
    {
        el.setValue("X");
        m_document->addElement(el);
    }
    if(el.value().Contains("{trans}"))
    {
        wxString trans = "";
        for(size_t i=0; i<m_atributes.GetCount(); i++)
        {
            if(m_atributes[i].name() == "trans")
            {
                trans = m_atributes[i].value();
                break;
            }
        }
        if(!trans.IsEmpty()) el.setValue("X");
        else el.setValue("");
        m_document->addElement(el);
    }
    if(el.value().Contains("{dapdph_forma}"))
    {
        wxString forma = "";
        for(size_t i=0; i<m_atributes.GetCount(); i++)
        {
            if(m_atributes[i].name() == "dapdph_forma")
            {
                forma = m_atributes[i].value();
                break;
            }
        }
        if(forma == "B")
        {
            el.setValue("X");
            m_document->addElement(el);
        }
        if(forma == "O")
        {
            el.setValue("X");
            m_document->addElement(el);
            TextElement el1(el.page(), el.value(), el.pos()+wxPoint(57,0), el.width(), el.alignment(), el.style());
            m_document->addElement(el1);
        }
        if(forma == "D")
        {
            el.setValue("X");
            el.setPos(el.pos()+wxPoint(26,0));
            m_document->addElement(el);
        }
        if(forma == "E")
        {
            el.setValue("X");
            el.setPos(el.pos()+wxPoint(26,0));
            m_document->addElement(el);
            TextElement el1(el.page(), el.value(), el.pos()+wxPoint(31,0), el.width(), el.alignment(), el.style());
            m_document->addElement(el1);
        }
    }
}

Dphdp3Printout::Dphdp3Printout(const wxString &title)
    : dxPrint(title), m_document(nullptr)
{
}

bool Dphdp3Printout::OnPrintPage(int page)
{
    wxDC* dc = GetDC();
    if (dc==nullptr || !dc->IsOk())
    {
        wxLogError(wxString::FromUTF8("Chyba při přípravě tisku, ukončuji tisk"));
        return false;
    }
    drawHeader(dc, m_mmToLogical);
    if(m_document)
    {
        dxElementsArray elements = m_document->elementsByPage(page);
        for(size_t i=0; i<elements.GetCount(); i++)
        {
            elements[i]->DrawToDC(dc, m_mmToLogical, -PAGE_OFFSET, -PAGE_OFFSET);
        }
    }
    drawFooter(page, dc, m_mmToLogical);
    return true;
}

void Dphdp3Printout::setDocument(Document *document)
{
    if(document)
    {
        m_document = document;
        m_footer = m_document->footer();
        m_footerText = m_document->footerText();
    }
}
