/***************************************************************
 * Name:      canvases.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/tokenzr.h>
#include "canvases.h"
#include "utils/dxutils.h"
#include "model/dbmodel.h"

BEGIN_EVENT_TABLE(TextCanvas, wxPanel)
    EVT_PAINT(TextCanvas::OnPaint)
END_EVENT_TABLE()

TextCanvas::TextCanvas(wxWindow *parent)
    : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxDOUBLE_BORDER|wxTAB_TRAVERSAL), m_element(TextElement())
{
    SetForegroundColour(*wxBLACK);
    SetBackgroundColour(*wxWHITE);
}

TextElement TextCanvas::element() const
{
    return m_element;
}

void TextCanvas::setElement(const TextElement &element)
{
    m_element = element;
}

void TextCanvas::SetAlignment(const wxTextAttrAlignment &alig)
{
    m_element.setAlignment(alig);
}

void TextCanvas::OnPaint(wxPaintEvent& /*event*/)
{
    wxPaintDC dc(this);
    dc.SetBackgroundMode(wxTRANSPARENT);
    wxPoint pos(IMAGE_Y_OFFSET, IMAGE_Y_OFFSET);
    dc.SetFont(m_element.style().font());
    dc.SetTextForeground(m_element.style().fontColour());
    dc.SetTextBackground(m_element.style().backColour());
    int textW, textH;
    textH = m_element.height(true);
    int width = dxutils::mmToPx(m_element.width(), &dc, true);
    if(m_element.style().backColour() != *wxWHITE)
    {
        dc.SetPen(*wxTRANSPARENT_PEN);
        dc.SetBrush(wxBrush(m_element.style().backColour()));
        dc.DrawRectangle(pos.x, pos.y, width, textH);
    }
    wxString wrappedLine, line;
    wxArrayString lines = wxStringTokenize(m_element.value(), wxT("\n") );
    wxArrayString wrappedLines;
    for(size_t l = 0; l < lines.GetCount(); l++)
    {
        line = lines[l];
        wrappedLine = "";
        size_t space = 0;
        for(size_t w = 0; w < line.Len(); w++)
        {
            wrappedLine += line[w];
            if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
            if(m_element.forceSpace()) space = w;
            dc.GetTextExtent(wrappedLine, &textW, &textH);
            if(textW > 0.9*width && space)
            {
                wrappedLines.Add(wrappedLine.Left(wrappedLine.Len()-w+space));
                wrappedLine = wxT("");
                w = space;
                space = 0;
            }
        }
        if(wrappedLine != wxEmptyString) wrappedLines.Add(wrappedLine);
    }
    double y = pos.y*1.0;
    for(size_t l = 0; l < wrappedLines.GetCount(); l++)
    {
        dc.GetTextExtent(wrappedLines[l], &textW, &textH);
        double cos = 0; // cell off-set
        switch(m_element.alignment())
        {
        case wxTEXT_ALIGNMENT_LEFT:
            cos = CELL_TEXT_OFFSET;
            break;
        case wxTEXT_ALIGNMENT_RIGHT:
            cos = width - textW - CELL_TEXT_OFFSET;
            break;
        default:
            cos = (width - textW) / 2;
        }
        dc.DrawText(wrappedLines[l], pos.x+wxRound(cos), wxRound(y));
        y += textH;
    }
}

BEGIN_EVENT_TABLE(LineCanvas, wxPanel)
    EVT_PAINT(LineCanvas::OnPaint)
END_EVENT_TABLE()

LineCanvas::LineCanvas(wxWindow* parent)
    : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxDOUBLE_BORDER|wxTAB_TRAVERSAL), m_element(LineElement())
{
    SetForegroundColour(*wxBLACK);
    SetBackgroundColour(*wxWHITE);
}

LineElement LineCanvas::element() const
{
    return m_element;
}

void LineCanvas::setElement(const LineElement &element)
{
    m_element = element;
}

void LineCanvas::OnPaint(wxPaintEvent& /*event*/)
{
    wxPaintDC dc(this);
    dc.SetBackgroundMode(wxTRANSPARENT);
    LineStyle style = m_element.lineStyle();
    dc.SetPen(wxPen(style.lineColour(), style.width(), style.style()));
    wxSize size = GetSize();
    int width = style.width();
    dc.DrawLine(2+width, size.y/2, size.x-2-width, size.y/2);
}

BEGIN_EVENT_TABLE(ImageCanvas, wxPanel)
    EVT_PAINT(ImageCanvas::OnPaint)
END_EVENT_TABLE()

ImageCanvas::ImageCanvas(wxWindow *parent)
    : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxDOUBLE_BORDER|wxTAB_TRAVERSAL)
{
    SetForegroundColour(*wxBLACK);
    SetBackgroundColour(*wxWHITE);
}

ImageElement ImageCanvas::element() const
{
    return m_element;
}

void ImageCanvas::setElement(const ImageElement &element)
{
    m_element = element;
}

void ImageCanvas::SetAlignment(const wxTextAttrAlignment &alig)
{
    m_element.setAlignment(alig);
}

void ImageCanvas::OnPaint(wxPaintEvent& /*event*/)
{
    if(m_element.isqrcode())
    {
        wxSize size = GetSize();
        wxSize imgsize = m_element.imagePxSize();
        int xpos = (size.x-imgsize.x)/2;
        int ypos = (size.y-imgsize.y)/2;
        wxPaintDC dc(this);
        dc.SetBackgroundMode(wxTRANSPARENT);
        QrCode qr = QrCode::encodeText(m_element.qrcodetxt().mb_str(),QrCode::Ecc::MEDIUM);
        int width = 6*QR_BORDER+qr.getSize()*QR_PIXEL*3;
        dc.SetFont(wxFont(wxFontInfo(9).FaceName(FACENAME)));
        int textW, textH;
        dc.GetTextExtent("QR Platba", &textW, &textH);
        dc.SetPen(*wxBLACK_PEN);
        dc.SetBrush(*wxTRANSPARENT_BRUSH);
        dc.DrawLine(xpos, ypos, xpos+width, ypos);
        dc.DrawLine(xpos, ypos, xpos, ypos+width);
        dc.DrawLine(xpos+width, ypos, xpos+width, ypos+width);
        dc.DrawLine(xpos, ypos+width, xpos+3*QR_BORDER-3*QR_MEZERA, ypos+width);
        dc.DrawLine(xpos+3*QR_BORDER+3*QR_MEZERA+textW, ypos+width, xpos+width, ypos+width);
        dc.DrawText("QR Platba", xpos+3*QR_BORDER, ypos+width);
        dc.SetPen(*wxTRANSPARENT_PEN);
        dc.SetBrush(*wxBLACK_BRUSH);
        for(int y=0; y<qr.getSize(); y++)
        {
            for(int x=0; x<qr.getSize(); x++)
            {
                const int color=qr.getModule(x, y);  // 0 for white, 1 for black
                if(0!=color)
                {
                    dc.DrawRectangle(xpos+3*QR_BORDER+3*QR_PIXEL*x, ypos+3*QR_BORDER+3*QR_PIXEL*y, 3*QR_PIXEL, 3*QR_PIXEL);
                }
            }
        }
        return;
    }
    wxString path = m_element.path();
    if(path == "{firma-logo}")
    {
        path = idbmodel->firmaLogo();
    }
    if(path == "{firma-razitko}")
    {
        path = idbmodel->firmaRazitko();
    }
    if(!wxFileExists(path))
    {
        return;
    }
    wxPaintDC dc(this);
    dc.SetBackgroundMode(wxTRANSPARENT);
    wxImage image;
    if(image.LoadFile(path))
    {
        wxSize size = GetSize();
        int ios = 0;
        switch(m_element.alignment())
        {
        case wxTEXT_ALIGNMENT_LEFT:
            ios = CELL_TEXT_OFFSET;
            break;
        case wxTEXT_ALIGNMENT_RIGHT:
            ios = size.x - image.GetWidth() - CELL_TEXT_OFFSET;
            break;
        default:
            ios = (size.x - image.GetWidth())/2;
        }
        dc.DrawBitmap(wxBitmap(image), ios, (size.y - image.GetHeight())/2);
    }
}

BEGIN_EVENT_TABLE(ParagraphCanvas, wxPanel)
    EVT_PAINT(ParagraphCanvas::OnPaint)
END_EVENT_TABLE()

ParagraphCanvas::ParagraphCanvas(wxWindow *parent)
    : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxDOUBLE_BORDER|wxTAB_TRAVERSAL)
{
    SetForegroundColour(*wxBLACK);
    SetBackgroundColour(*wxWHITE);
}

TextParagraphElement ParagraphCanvas::element() const
{
    return m_element;
}

void ParagraphCanvas::setElement(const TextParagraphElement &element)
{
    m_element = element;
}

void ParagraphCanvas::SetAlignment(const wxTextAttrAlignment &alig)
{
    m_element.setAlignment(alig);
}

void ParagraphCanvas::OnPaint(wxPaintEvent& /*event*/)
{
    wxPaintDC dc(this);
    dc.SetBackgroundMode(wxTRANSPARENT);
    dc.SetFont(m_element.style().font());
    wxPoint pos(IMAGE_Y_OFFSET, IMAGE_Y_OFFSET);
    int textW, textH;
    textH = m_element.height(true);
    int width = dxutils::mmToPx(m_element.width(), &dc, true);
    if(m_element.style().backColour() != *wxWHITE)
    {
        dc.SetPen(*wxTRANSPARENT_PEN);
        dc.SetBrush(wxBrush(m_element.style().backColour()));
        dc.DrawRectangle(pos.x, pos.y, width, textH);
    }
    if(m_element.border())
    {
        dc.SetPen(wxPen(m_element.borderStyle().lineColour(), m_element.borderStyle().width(), m_element.borderStyle().style()));
        dc.SetBrush(*wxTRANSPARENT_BRUSH);
        dc.DrawRectangle(pos.x, pos.y, width, textH);
    }
    dc.SetTextForeground(m_element.style().fontColour());
    dc.SetTextBackground(m_element.style().backColour());
    wxString wrappedLine, line;
    wxArrayString lines;
    wxArrayString value = m_element.value();
    for(size_t i=0; i<value.GetCount(); i++)
    {
        wxArrayString tok = wxStringTokenize(value[i], wxT("\n"));
        for(size_t j=0; j<tok.GetCount(); j++)
        {
            lines.Add(tok[j]);
        }
    }
    wxArrayString wrappedLines;
    for(size_t l = 0; l < lines.GetCount(); l++)
    {
        line = lines[l];
        wrappedLine = "";
        size_t space = 0;
        for(size_t w = 0; w < line.Len(); w++)
        {
            wrappedLine += line[w];
            if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
            dc.GetTextExtent(wrappedLine, &textW, &textH);
            if(textW > 0.9*width && space)
            {
                wrappedLines.Add(wrappedLine.Left(wrappedLine.Len()-w+space));
                wrappedLine = wxT("");
                w = space;
                space = 0;
            }
        }
        if(wrappedLine != wxEmptyString) wrappedLines.Add(wrappedLine);
    }
    double y = pos.y*1.0;
    for(size_t l = 0; l < wrappedLines.GetCount(); l++)
    {
        dc.GetTextExtent(wrappedLines[l], &textW, &textH);
        double cos = 0; // cell off-set
        switch(m_element.alignment())
        {
        case wxTEXT_ALIGNMENT_LEFT:
            cos = CELL_TEXT_OFFSET;
            break;
        case wxTEXT_ALIGNMENT_RIGHT:
            cos = width - textW - CELL_TEXT_OFFSET;
            break;
        default:
            cos = (width - textW) / 2;
        }
        dc.DrawText(wrappedLines[l], pos.x+wxRound(cos), wxRound(y));
        y += textH;
    }
}

BEGIN_EVENT_TABLE(RowCanvas, wxPanel)
    EVT_PAINT(RowCanvas::OnPaint)
END_EVENT_TABLE()

RowCanvas::RowCanvas(wxWindow *parent)
    : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxDOUBLE_BORDER|wxTAB_TRAVERSAL)
{
    SetForegroundColour(*wxBLACK);
    SetBackgroundColour(*wxWHITE);
}

TableRowElement RowCanvas::element() const
{
    return m_element;
}

void RowCanvas::setElement(const TableRowElement &element)
{
    m_element = element;
}

void RowCanvas::OnPaint(wxPaintEvent& /*event*/)
{
    if((m_element.value().GetCount() != m_element.width().GetCount()) && (m_element.value().GetCount() != m_element.alignment().GetCount()))
        return;
    wxPaintDC dc(this);
    dc.SetBackgroundMode(wxTRANSPARENT);
    wxPoint pos(IMAGE_Y_OFFSET, IMAGE_Y_OFFSET);
    dc.SetFont(m_element.style().font());
    int height = m_element.height(true);
    wxArrayInt widths = m_element.width();
    for(size_t i=0; i < widths.GetCount(); i++)
    {
        widths[i] = dxutils::mmToPx(widths[i], &dc, true);
    }
    int width = dxutils::mmToPx(m_element.totalWidth(), &dc, true);
    if(m_element.style().backColour() != *wxWHITE)
    {
        dc.SetPen(*wxTRANSPARENT_PEN);
        dc.SetBrush(wxBrush(m_element.style().backColour()));
        dc.DrawRectangle(pos.x, pos.y, width, height);
    }
    if(m_element.border())
    {
        dc.SetPen(wxPen(m_element.borderStyle().lineColour(), m_element.borderStyle().width(), m_element.borderStyle().style()));
        dc.SetBrush(*wxTRANSPARENT_BRUSH);
        dc.DrawLine(pos.x, pos.y, pos.x+width, pos.y);
        dc.DrawLine(pos.x, pos.y+height, pos.x+width, pos.y+height);
        width = 0;
        for(size_t i=0; i<widths.GetCount(); i++)
        {
            dc.DrawLine(pos.x+width, pos.y, pos.x+width, pos.y+height);
            width += widths[i];
        }
        dc.DrawLine(pos.x+width, pos.y, pos.x+width, pos.y+height);
    }
    dc.SetTextForeground(m_element.style().fontColour());
    dc.SetTextBackground(m_element.style().backColour());
    wxArrayString values = m_element.value();
    wxArrayInt alignments = m_element.alignment();
    for(size_t i=0; i<values.GetCount(); i++)
    {
        int textW, textH;
        wxString wrappedLine, line;
        wxArrayString lines = wxStringTokenize(values[i], wxT("\n"));
        wxArrayString wrappedLines;
        for(size_t l = 0; l < lines.GetCount(); l++)
        {
            line = lines[l];
            wrappedLine = "";
            size_t space = 0;
            for(size_t w = 0; w < line.Len(); w++)
            {
                wrappedLine += line[w];
                if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
                dc.GetTextExtent(wrappedLine, &textW, &textH);
                if(textW > 0.9*widths[i] && space)
                {
                    wrappedLines.Add(wrappedLine.Left(wrappedLine.Len()-w+space));
                    wrappedLine = wxT("");
                    w = space;
                    space = 0;
                }
            }
            if(wrappedLine != wxEmptyString) wrappedLines.Add(wrappedLine);
        }
        int x = pos.x;
        if(i)
        {
            for(size_t w = 0; w <= (i-1); w++)
            {
                x += widths[w];
            }
        }
        int y = pos.y;
        for(size_t l = 0; l < wrappedLines.GetCount(); l++)
        {
            dc.GetTextExtent(wrappedLines[l], &textW, &textH);
            double cos = 0; // cell off-set
            switch(alignments[i])
            {
            case wxTEXT_ALIGNMENT_LEFT:
                cos = CELL_TEXT_OFFSET;
                break;
            case wxTEXT_ALIGNMENT_RIGHT:
                cos = widths[i] - textW - CELL_TEXT_OFFSET;
                break;
            default:
                cos = (widths[i] - textW) / 2;
            }
            dc.DrawText(wrappedLines[l], x+wxRound(cos), y);
            y += textH;
        }
    }
}

BEGIN_EVENT_TABLE(RectangleCanvas, wxPanel)
    EVT_PAINT(RectangleCanvas::OnPaint)
END_EVENT_TABLE()

RectangleCanvas::RectangleCanvas(wxWindow* parent)
    : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxDOUBLE_BORDER|wxTAB_TRAVERSAL), m_element(RectangleElement())
{
    SetForegroundColour(*wxBLACK);
    SetBackgroundColour(*wxWHITE);
}

RectangleElement RectangleCanvas::element() const
{
    return m_element;
}

void RectangleCanvas::setElement(const RectangleElement &element)
{
    m_element = element;
}

void RectangleCanvas::OnPaint(wxPaintEvent& /*event*/)
{
    wxPaintDC dc(this);
    dc.SetBackgroundMode(wxTRANSPARENT);
    LineStyle style = m_element.lineStyle();
    dc.SetPen(wxPen(style.lineColour(), style.width(), style.style()));
    wxSize size = GetSize();
    if(m_element.fillet()) dc.DrawRoundedRectangle(IMAGE_X_OFFSET, IMAGE_Y_OFFSET, size.x-2*IMAGE_X_OFFSET, size.y-2*IMAGE_Y_OFFSET, m_element.fillet());
    else dc.DrawRectangle(IMAGE_X_OFFSET, IMAGE_Y_OFFSET, size.x-2*IMAGE_X_OFFSET, size.y-2*IMAGE_Y_OFFSET);
}
