/***************************************************************
 * Name:      rowdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/fontdata.h>
#include <wx/fontdlg.h>
#include <wx/colordlg.h>
#include "rowdialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/delete.xpm"
#include "tags.h"
#include "model/dbmodel.h"

class CellDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(CellDialog)
    DECLARE_EVENT_TABLE()
public:
    CellDialog() : m_mainSizer(nullptr), m_cellWidth(nullptr), m_text(nullptr), m_alignment(nullptr), m_keyword(nullptr) {}
    CellDialog(wxWindow *parent, const wxString &text, int width, wxTextAttrAlignment alignment);

    wxString text() const;
    int width() const;
    wxTextAttrAlignment alignment() const;
private:
    wxBoxSizer *m_mainSizer;
    wxSpinCtrl *m_cellWidth;
    wxTextCtrl *m_text;
    wxChoice *m_alignment;
    wxButton *m_keyword;

    void OnKeyword(wxCommandEvent& event);
};

IMPLEMENT_DYNAMIC_CLASS(CellDialog, wxDialog)

BEGIN_EVENT_TABLE(CellDialog, wxDialog)
    EVT_BUTTON(ID_ROW_KEYWORD, CellDialog::OnKeyword)
END_EVENT_TABLE()

CellDialog::CellDialog(wxWindow* parent, const wxString& text, int width, wxTextAttrAlignment alignment)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti buňky"))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    m_text = new wxTextCtrl(this, wxID_ANY);
    m_text->SetValue(text);
    gridSizer->Add(m_text, wxGBPosition(0,0), wxGBSpan(1,2), wxEXPAND);
    m_keyword = new wxButton(this, ID_ROW_KEYWORD, "...");
    m_keyword->SetToolTip(wxString::FromUTF8("Vložit klíčové slovo"));
    gridSizer->Add(m_keyword, wxGBPosition(0,2), wxDefaultSpan);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Šířka buňky")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cellWidth = new wxSpinCtrl(this, wxID_ANY);
    m_cellWidth->SetRange(0,210);
    m_cellWidth->SetValue(width);
    gridSizer->Add(m_cellWidth, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zarovnání")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_alignment = new wxChoice(this, ID_TEXT_ALIGNMENT);
    m_alignment->Append("Doleva");
    m_alignment->Append(wxString::FromUTF8("Na střed"));
    m_alignment->Append("Doprava");
    m_alignment->SetSelection(alignment-1);
    gridSizer->Add(m_alignment, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    m_mainSizer->Add(gridSizer, 1, wxALL, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

wxString CellDialog::text() const
{
    return m_text->GetValue();
}

int CellDialog::width() const
{
    return m_cellWidth->GetValue();
}

wxTextAttrAlignment CellDialog::alignment() const
{
    return static_cast<wxTextAttrAlignment>(m_alignment->GetSelection()+1);
}

void CellDialog::OnKeyword(wxCommandEvent& /*event*/)
{
    wxArrayString choices;
    for(size_t i=0; i<numPolozkaTags; i++)
    {
        choices.Add(polozkaTags[i]);
    }
    if(idbmodel->isFirmaPlatce())
    {
        for(size_t i=0; i<numSazbaTags; i++)
        {
            choices.Add(sazbaTags[i]);
        }
    }
    wxSingleChoiceDialog vyber(this, wxString::FromUTF8("Vyberte klíčové slovo"), wxString::FromUTF8("Klíčové slovo"), choices);
    if(vyber.ShowModal() == wxID_OK)
    {
        m_text->AppendText(vyber.GetStringSelection());
    }
}

IMPLEMENT_DYNAMIC_CLASS(RowDialog, wxDialog)

BEGIN_EVENT_TABLE(RowDialog, wxDialog)
    EVT_BUTTON(wxID_OK, RowDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, RowDialog::OnCancel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_ROW_LIST, RowDialog::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_ROW_LIST, RowDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_ROW_LIST, RowDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_ROW_LIST, RowDialog::OnItemDeselected)
    EVT_MENU(ID_ROW_ADD, RowDialog::OnAddCell)
    EVT_MENU(ID_ROW_EDIT, RowDialog::OnEditCell)
    EVT_MENU(ID_ROW_COPY, RowDialog::OnCopyCell)
    EVT_MENU(ID_ROW_DELETE, RowDialog::OnDeleteCell)
    EVT_BUTTON(ID_ROW_FONT, RowDialog::OnFont)
    EVT_BUTTON(ID_ROW_FONTCOLOR, RowDialog::OnFontColor)
    EVT_BUTTON(ID_ROW_BACKCOLOR, RowDialog::OnBackColor)
    EVT_CHECKBOX(ID_ROW_BORDER, RowDialog::OnBorder)
    EVT_CHOICE(ID_ROW_BORDERSTYLE, RowDialog::OnBorderStyle)
    EVT_BUTTON(ID_ROW_BORDERCOLOR, RowDialog::OnBorderColor)
    EVT_SPINCTRL(ID_ROW_BORDERWIDTH, RowDialog::OnBorderWidth)
END_EVENT_TABLE()

//Novy text
RowDialog::RowDialog(wxWindow *parent, const TextStyle &style)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti řádku tabulky")), m_selectedId(-1)
{
    wxArrayString values;
    values.Add("Text1");
    values.Add("Text2");
    wxArrayInt widths;
    widths.Add(100);
    widths.Add(100);
    wxArrayInt alignments;
    alignments.Add(wxTEXT_ALIGNMENT_LEFT);
    alignments.Add(wxTEXT_ALIGNMENT_LEFT);
    m_element = TableRowElement(1, values, wxPoint(0,0), widths, alignments, style);
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

//Uprava textu
RowDialog::RowDialog(wxWindow *parent, const TableRowElement &element)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti řádku tabulky")), m_element(element),
      m_selectedId(-1)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

TableRowElement RowDialog::element() const
{
    return m_element;
}

void RowDialog::setElement(const TableRowElement &element)
{
    m_element = element;
}

void RowDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sekce"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_section = new wxChoice(this, ID_LINE_SECTION);
    m_section->Append(wxString::FromUTF8("Hlavička 1"));
    m_section->Append(wxString::FromUTF8("Hlavička 2"));
    m_section->Append(wxString::FromUTF8("Hlavička 3"));
    m_section->Append(wxString::FromUTF8("Položky"));
    m_section->Append("Celkem");
    m_section->Append("DPH");
    m_section->Append(wxString::FromUTF8("Patička 3"));
    m_section->Append(wxString::FromUTF8("Patička 2"));
    m_section->Append(wxString::FromUTF8("Patička 1"));
    m_section->Append(wxString::FromUTF8("Žádná"));
    m_section->SetSelection(10);
    gridSizer->Add(m_section, wxGBPosition(0,1), wxGBSpan(1,3), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pozice odstavce - Xová souřadnice")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_posX = new wxSpinCtrl(this);
    m_posX->SetRange(0,210);
    gridSizer->Add(m_posX, wxGBPosition(1,1), wxGBSpan(1,3), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pozice odstavce - Yová souřadnice")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_posY = new wxSpinCtrl(this);
    m_posY->SetRange(0,297);
    gridSizer->Add(m_posY, wxGBPosition(2,1), wxGBSpan(1,3), wxEXPAND);
    wxStaticBoxSizer *box = new wxStaticBoxSizer(wxVERTICAL, this, "Okraje");
    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_ROW_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat buňku"));
    m_bar->AddTool(ID_ROW_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit buňku"));
    m_bar->AddTool(ID_ROW_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat buňku"));
    m_bar->AddTool(ID_ROW_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat buňku"));
    m_bar->Realize();
    sizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    m_cells = new wxListCtrl(this, ID_ROW_LIST, wxDefaultPosition, wxDefaultSize, wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText("Text");
    col0.SetWidth(350);
    m_cells->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText(wxString::FromUTF8("Šířka"));
    col1.SetWidth(80);
    m_cells->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText(wxString::FromUTF8("Zarovnání"));
    col2.SetWidth(100);
    m_cells->InsertColumn(2, col2);
    sizer->Add(m_cells, 1, wxALL|wxEXPAND, 5);
    box->Add(sizer, 1, wxGROW);
    gridSizer->Add(box, wxGBPosition(3,0), wxGBSpan(1,4), wxEXPAND);
    m_fontSelect = new wxButton(this, ID_ROW_FONT, "Font");
    gridSizer->Add(m_fontSelect, wxGBPosition(4,1), wxDefaultSpan);
    m_fontColorSelect = new wxButton(this, ID_ROW_FONTCOLOR, "Barva textu");
    gridSizer->Add(m_fontColorSelect, wxGBPosition(4,2), wxDefaultSpan);
    m_backColorSelect = new wxButton(this, ID_ROW_BACKCOLOR, wxString::FromUTF8("Barva pozadí"));
    gridSizer->Add(m_backColorSelect, wxGBPosition(4,3), wxDefaultSpan);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Okraj"), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_border = new wxCheckBox(this, ID_ROW_BORDER, wxEmptyString);
    gridSizer->Add(m_border, wxGBPosition(5,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Styl okraje"), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_borderStyle = new wxChoice(this, ID_ROW_BORDERSTYLE);
    m_borderStyle->Append(wxString::FromUTF8("Plná čára"));
    m_borderStyle->Append(wxString::FromUTF8("Tečkovaná čára"));
    m_borderStyle->Append(wxString::FromUTF8("Dlouze čárkovaná čára"));
    m_borderStyle->Append(wxString::FromUTF8("Krátce čárkovaná čára"));
    m_borderStyle->Append(wxString::FromUTF8("Čerchovaná čára"));
    m_borderStyle->SetSelection(0);
    m_borderStyle->Enable(false);
    gridSizer->Add(m_borderStyle, wxGBPosition(6,1), wxGBSpan(1,3), wxEXPAND);
    m_borderColor = new wxButton(this, ID_ROW_BORDERCOLOR, "Barva okraje");
    m_borderColor->Enable(false);
    gridSizer->Add(m_borderColor, wxGBPosition(7,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Šířka okraje")), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_borderWidth = new wxSpinCtrl(this, ID_ROW_BORDERWIDTH);
    m_borderWidth->SetRange(1,20);
    m_borderWidth->Enable(false);
    gridSizer->Add(m_borderWidth, wxGBPosition(8,1), wxGBSpan(1,3), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Náhled")), wxGBPosition(9,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mainSizer->Add(gridSizer, 1, wxLEFT|wxTOP|wxRIGHT, 5);
    m_preview = new RowCanvas(this);
    m_mainSizer->Add(m_preview, 0, wxALL, 2);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void RowDialog::fillControls()
{
    m_posX->SetValue(m_element.pos().x);
    m_posY->SetValue(m_element.pos().y);
    fillList();
    m_border->SetValue(m_element.border());
    wxCommandEvent ev;
    OnBorder(ev);
    LineStyle style = m_element.borderStyle();
    m_borderWidth->SetValue(style.width());
    m_borderStyle->SetSelection(style.style()-100);
    setPreviewSize();
    m_preview->setElement(m_element);
    m_section->SetSelection(m_element.section());
}

void RowDialog::OnOk(wxCommandEvent& /*event*/)
{
    m_element.setPage(1);
    m_element.setPos(wxPoint(m_posX->GetValue(), m_posY->GetValue()));
    m_element.setBorder(m_border->IsChecked());
    LineStyle style = m_element.borderStyle();
    style.setWidth(m_borderWidth->GetValue());
    style.setStyle(static_cast<wxPenStyle>(m_borderStyle->GetSelection()+100));
    m_element.setBorderStyle(style);
    m_element.setSection(static_cast<DOCSECTION>(m_section->GetSelection()));
    EndModal(wxID_OK);
}

void RowDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void RowDialog::OnFont(wxCommandEvent& /*event*/)
{
    wxFontData data;
    data.SetInitialFont(m_element.style().font());
    data.SetColour(m_element.style().fontColour());
    wxFontDialog dialog(this, data);
    if(dialog.ShowModal() == wxID_OK)
    {
        wxFontData retData = dialog.GetFontData();
        TextStyle style = m_element.style();
        style.setFont(retData.GetChosenFont());
        style.setFontColour(retData.GetColour());
        m_element.setStyle(style);
        setPreviewSize();
        m_preview->setElement(m_element);
        m_mainSizer->Layout();
        m_mainSizer->Fit(this);
        m_mainSizer->SetSizeHints(this);
        m_preview->Refresh();
    }
}

void RowDialog::OnFontColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_element.style().fontColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle("Vyberte barvu fontu");
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        TextStyle style = m_element.style();
        style.setFontColour(data.GetColour());
        m_element.setStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void RowDialog::OnBackColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_element.style().backColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle(wxString::FromUTF8("Vyberte barvu pozadí"));
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        TextStyle style = m_element.style();
        style.setBackColour(data.GetColour());
        m_element.setStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void RowDialog::OnBorder(wxCommandEvent& /*event*/)
{
    if(m_border->IsChecked())
    {
        m_borderStyle->Enable(true);
        m_borderWidth->Enable(true);
        m_borderColor->Enable(true);
    }
    else
    {
        m_borderStyle->Enable(false);
        m_borderWidth->Enable(false);
        m_borderColor->Enable(false);
    }
    m_element.setBorder(m_border->IsChecked());
    m_preview->setElement(m_element);
    m_preview->Refresh();
}

void RowDialog::OnBorderStyle(wxCommandEvent& /*event*/)
{
    LineStyle style = m_element.borderStyle();
    style.setStyle(static_cast<wxPenStyle>(m_borderStyle->GetSelection()+100));
    m_element.setBorderStyle(style);
    m_preview->setElement(m_element);
    m_preview->Refresh();
}

void RowDialog::OnBorderWidth(wxSpinEvent& /*event*/)
{
    LineStyle style = m_element.borderStyle();
    style.setWidth(m_borderWidth->GetValue());
    m_element.setBorderStyle(style);
    m_preview->setElement(m_element);
    m_preview->Refresh();
}

void RowDialog::OnBorderColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_element.borderStyle().lineColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle("Vyberte barvu okraje");
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        LineStyle style = m_element.borderStyle();
        style.setLineColour(data.GetColour());
        m_element.setBorderStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void RowDialog::setPreviewSize()
{
    wxScreenDC dc;
    int width = dxutils::mmToPx(m_element.totalWidth(), &dc, true);
    m_preview->SetMinSize(wxSize(width,m_element.height(true))+wxSize(2*IMAGE_Y_OFFSET,2*IMAGE_Y_OFFSET));
    m_preview->SetSize(wxSize(width,m_element.height(true))+wxSize(2*IMAGE_Y_OFFSET,2*IMAGE_Y_OFFSET));
}

void RowDialog::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
        m_selectedId = selectedIndex;
    enableTools();
}

void RowDialog::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    wxMenu popup;
    popup.Append(ID_ROW_ADD, wxString::FromUTF8("Přidat buňku"));
    if(m_selectedId != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_ROW_EDIT, wxString::FromUTF8("Upravit buňku"));
        popup.Append(ID_ROW_COPY, wxString::FromUTF8("Kopírovat buňku"));
        popup.Append(ID_ROW_DELETE, wxString::FromUTF8("Smazat buňku"));
    }
    PopupMenu(&popup, event.GetPoint()+m_cells->GetPosition());
}

void RowDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    wxCommandEvent dummy;
    OnEditCell(dummy);
}

void RowDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void RowDialog::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void RowDialog::OnEditCell(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    CellDialog dialog(this, m_element.cellValue(static_cast<size_t>(m_selectedId)), m_element.cellWidth(static_cast<size_t>(m_selectedId)), m_element.cellAlignment(static_cast<size_t>(m_selectedId)));
    if(dialog.ShowModal() == wxID_OK)
    {
        m_element.setCellValue(static_cast<size_t>(m_selectedId), dialog.text());
        m_element.setCellWidth(static_cast<size_t>(m_selectedId), dialog.width());
        m_element.setCellAlignment(static_cast<size_t>(m_selectedId), dialog.alignment());
        fillList();
    }
}

void RowDialog::OnCopyCell(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    CellDialog dialog(this, m_element.cellValue(static_cast<size_t>(m_selectedId)), m_element.cellWidth(static_cast<size_t>(m_selectedId)), m_element.cellAlignment(static_cast<size_t>(m_selectedId)));
    if(dialog.ShowModal() == wxID_OK)
    {
        m_element.addCellValue(dialog.text());
        m_element.addCellWidth(dialog.width());
        m_element.addCellAlignment(dialog.alignment());
        fillList();
    }
}

void RowDialog::OnAddCell(wxCommandEvent& /*event*/)
{
    CellDialog dialog(this, "Text", 50, wxTEXT_ALIGNMENT_LEFT);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_element.addCellValue(dialog.text());
        m_element.addCellWidth(dialog.width());
        m_element.addCellAlignment(dialog.alignment());
        fillList();
    }
}

void RowDialog::OnDeleteCell(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat buňku %s?"), m_element.cellValue(static_cast<size_t>(m_selectedId))),
            "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_element.deleteCell(static_cast<size_t>(m_selectedId));
        fillList();
    }
}

void RowDialog::enableTools()
{
    if(m_selectedId == -1)
    {
        m_bar->EnableTool(ID_ROW_EDIT, false);
        m_bar->EnableTool(ID_ROW_COPY, false);
        m_bar->EnableTool(ID_ROW_DELETE, false);
    }
    else
    {
        m_bar->EnableTool(ID_ROW_EDIT, true);
        m_bar->EnableTool(ID_ROW_COPY, true);
        m_bar->EnableTool(ID_ROW_DELETE, true);
    }
}

void RowDialog::fillList()
{
    if((m_element.value().GetCount() != m_element.width().GetCount()) && (m_element.value().GetCount() != m_element.alignment().GetCount()))
        return;
    m_cells->DeleteAllItems();
    m_cells->Hide();
    wxArrayString values = m_element.value();
    wxArrayInt widths = m_element.width();
    wxArrayInt alignments = m_element.alignment();
    for(size_t i=0; i < values.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<int>(i));
        item.SetText(values[i]);
        m_cells->InsertItem(item);
        m_cells->SetItem(static_cast<int>(i), 1, wxString::Format("%d", widths[i]));
        switch(alignments[i]) {
        case wxTEXT_ALIGNMENT_LEFT: m_cells->SetItem(static_cast<int>(i), 2, "Doleva"); break;
        case wxTEXT_ALIGNMENT_RIGHT: m_cells->SetItem(static_cast<int>(i), 2, "Doprava"); break;
        default: m_cells->SetItem(static_cast<int>(i), 2, wxString::FromUTF8("Na střed"));
        }
    }
    m_cells->Show();
    if(values.GetCount())
        m_cells->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    else
        setSelectedId(-1);
    setPreviewSize();
    m_preview->setElement(m_element);
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_preview->Refresh();
}
