/***************************************************************
 * Name:      dxprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/log.h>
#include "wx/pdfdc.h"
#include <wx/dcscreen.h>
#include "dxprint.h"
#include "dxdefs.h"

dxPrint::dxPrint(const wxString &title)
    : wxPrintout(title), m_paperSize(wxPAPER_A4), m_paperOrientation(wxPORTRAIT), m_pages(1), m_marginLeft(20-PAGE_OFFSET), m_marginRight(20+PAGE_OFFSET),
      m_marginTop(20-PAGE_OFFSET), m_marginBottom(20), m_footer(F_NUMBERING), m_footerText(""), m_headerLeftText(""), m_headerCenterText(""),
      m_headerRightText(""), m_headerLine(false), m_mmToLogical(0.0)
{
//    performPageSetup();
}

void dxPrint::OnBeginPrinting()
{
    wxDC* dc = GetDC();
    if (dc==nullptr || !dc->IsOk())
    {
        wxLogError(wxString::FromUTF8("Chyba při přípravě tisku, ukončuji tisk"));
        return;
    }
    if(dc->IsKindOf(CLASSINFO(wxPdfDC)))
    {
        wxSize devicePPI = dc->GetPPI();
        int ppiPrinterX, ppiPrinterY;
        ppiPrinterX = devicePPI.GetWidth();
        ppiPrinterY = devicePPI.GetHeight();
        int ppiScreenY;
        wxScreenDC sdc;
        ppiScreenY = sdc.GetPPI().GetHeight();
        double scale = (ppiPrinterY/static_cast<double>(ppiScreenY));
        dc->SetUserScale(scale, scale);
        m_mmToLogical = (ppiPrinterX/(scale*25.4));
    }
    else
    {
        // Get the logical pixels per inch of screen and printer
        int ppiScreenX, ppiScreenY;
        GetPPIScreen(&ppiScreenX, &ppiScreenY);
        int ppiPrinterX, ppiPrinterY;
        GetPPIPrinter(&ppiPrinterX, &ppiPrinterY);
        // This scales the DC so that the printout roughly represents the screen
        // scaling. The text point size _should_ be the right size but in fact is
        // too small for some reason. This is a detail that will need to be
        // addressed at some point but can be fudged for the moment.
        double scale = (static_cast<double>(ppiPrinterX)/static_cast<double>(ppiScreenX));
        // Now we have to check in case our real page size is reduced (e.g. because
        // we're drawing to a print preview memory DC)
        int pageWidth, pageHeight;
        int w, h;
        dc->GetSize(&w, &h);
        GetPageSizePixels(&pageWidth, &pageHeight);
        // If printer pageWidth == current DC width, then this doesn't change. But w
        // might be the preview bitmap width, so scale down.
        double overallScale = scale * static_cast<double>((w/static_cast<double>(pageWidth)));
        dc->SetUserScale(overallScale, overallScale);
        // Calculate conversion factor for converting millimetres into logical
        // units. There are approx. 25.4 mm to the inch. There are ppi device units
        // to the inch. Therefore 1 mm corresponds to ppi/25.4 device units. We also
        // divide by the screen-to-printer scaling factor, because we need to
        // unscale to pass logical units to DrawLine.
        m_mmToLogical = static_cast<double>((ppiPrinterX/(scale*25.4)));
    }
}

bool dxPrint::OnPrintPage(int page)
{
    wxDC* dc = GetDC();
    if (dc==nullptr || !dc->IsOk())
    {
        wxLogError(wxString::FromUTF8("Chyba při přípravě tisku, ukončuji tisk"));
        return false;
    }
    drawHeader(dc, m_mmToLogical);
    drawFooter(page, dc, m_mmToLogical);
    return true;
}

bool dxPrint::HasPage(int page)
{
    return page >= 1 && page <= m_pages;
}

bool dxPrint::OnBeginDocument(int startPage, int endPage)
{
    return wxPrintout::OnBeginDocument(startPage, endPage);
}

void dxPrint::GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo)
{
    *minPage = 1;
    *maxPage = m_pages;
    *selPageFrom = 1;
    *selPageTo = m_pages;
}

void dxPrint::performPageSetup()
{
    wxPrintData printdata;
    printdata.SetPrintMode(wxPRINT_MODE_PRINTER);
    printdata.SetOrientation(m_paperOrientation);
    printdata.SetNoCopies(1);
    printdata.SetPaperId(m_paperSize);
    m_pageSetup = wxPageSetupDialogData(printdata);
#ifdef __WXMAC__
    m_pageSetup.SetMarginTopLeft(wxPoint(0, 0));
    m_pageSetup.SetMarginBottomRight(wxPoint(0, 0));
#else
    m_pageSetup.SetMarginTopLeft(wxPoint(m_marginLeft, m_marginTop));
    m_pageSetup.SetMarginBottomRight(wxPoint(m_marginRight, m_marginBottom));
#endif
}

wxPrintData dxPrint::printData() const
{
    return m_pageSetup.GetPrintData();
}

int dxPrint::pages() const
{
    return m_pages;
}

void dxPrint::setPages(int pages)
{
    m_pages = pages;
}

wxString dxPrint::headerLeftText() const
{
    return m_headerLeftText;
}

void dxPrint::setHeaderLeftText(const wxString &headerLeftText)
{
    m_headerLeftText = headerLeftText;
}

wxString dxPrint::headerCenterText() const
{
    return m_headerCenterText;
}

void dxPrint::setHeaderCenterText(const wxString &headerCenterText)
{
    m_headerCenterText = headerCenterText;
}

wxString dxPrint::headerRightText() const
{
    return m_headerRightText;
}

void dxPrint::setHeaderRightText(const wxString &headerRightText)
{
    m_headerRightText = headerRightText;
}

bool dxPrint::headerLine() const
{
    return m_headerLine;
}

void dxPrint::setHeaderLine(bool headerLine)
{
    m_headerLine = headerLine;
}

int dxPrint::marginBottom() const
{
    return m_marginBottom;
}

void dxPrint::setMarginBottom(int marginBottom)
{
    m_marginBottom = marginBottom;
}

dxPrint::Footer dxPrint::footer() const
{
    return m_footer;
}

void dxPrint::setFooter(const Footer &footer)
{
    m_footer = footer;
}

wxString dxPrint::footerText() const
{
    return m_footerText;
}

void dxPrint::setFooterText(const wxString &footerText)
{
    m_footerText = footerText;
}

wxPrintOrientation dxPrint::paperOrientation() const
{
    return m_paperOrientation;
}

void dxPrint::setPaperOrientation(const wxPrintOrientation &paperOrientation)
{
    m_paperOrientation = paperOrientation;
}

int dxPrint::marginTop() const
{
    return m_marginTop;
}

void dxPrint::setMarginTop(int marginTop)
{
    m_marginTop = marginTop;
}

int dxPrint::marginRight() const
{
    return m_marginRight;
}

void dxPrint::setMarginRight(int marginRight)
{
    m_marginRight = marginRight;
}

int dxPrint::marginLeft() const
{
    return m_marginLeft;
}

void dxPrint::setMarginLeft(int marginLeft)
{
    m_marginLeft = marginLeft;
}

void dxPrint::drawFooter(int page, wxDC *dc, double mmToLogical)
{
    if(m_footer == F_NONE)
        return;
    wxString text = m_footer==F_TEXT?m_footerText:wxString::Format(wxString::FromUTF8("Stránka %d z %d"), page, m_pages);
    if(text.IsEmpty())
        return;
    int pageWidthMM, pageHeightMM;
    int ydist = 0, xdist = 0;
    if(dc->IsKindOf(CLASSINFO(wxPdfDC)))
    {
        pageWidthMM = m_pageSetup.GetPaperSize().x;
        pageHeightMM = m_pageSetup.GetPaperSize().y;
        ydist = PAGE_OFFSET;
        xdist = PAGE_OFFSET;
    }
    else
    {
        GetPageSizeMM(&pageWidthMM, &pageHeightMM);
    }
    wxSize textSize = dc->GetTextExtent(text);
    int leftMargin = 10+xdist;
    int rightMargin = 10;
    int bottomMargin = 10-PAGE_OFFSET+ydist;
    double bottomMarginLogical = (mmToLogical*(pageHeightMM-bottomMargin));
    double xPos = (((((pageWidthMM - leftMargin - rightMargin)/2.0)+leftMargin)*mmToLogical) - (textSize.x/2.0));
    dc->SetFont(wxFont(wxFontInfo(10).FaceName(FACENAME)));
    dc->DrawText(text, static_cast<int>(xPos), static_cast<int>(bottomMarginLogical));
}

void dxPrint::drawHeader(wxDC *dc, double mmToLogical)
{
    int pageWidthMM, pageHeightMM;
    int ydist = 0, xdist = 0;
    if(dc->IsKindOf(CLASSINFO(wxPdfDC)))
    {
        pageWidthMM = m_pageSetup.GetPaperSize().x;
        pageHeightMM = m_pageSetup.GetPaperSize().y;
        ydist = PAGE_OFFSET;
        xdist = PAGE_OFFSET;
    }
    else
    {
        GetPageSizeMM(&pageWidthMM, &pageHeightMM);
    }
    //TODO hnusnej hack aby pri Gtk3 nebylo preview cele cerne
#ifdef __WXGTK3__
    if(1)
    {
        dc->SetBrush(*wxWHITE_BRUSH);
        dc->DrawRectangle(-1,-1,pageWidthMM*mmToLogical,pageHeightMM*mmToLogical);
        dc->SetBrush(*wxTRANSPARENT_BRUSH);
    }
#endif
    wxSize textSize;
    int leftMargin = 10+xdist;
    int rightMargin = 10;
    int topMargin = 10-PAGE_OFFSET+ydist;
    double topMarginLogical = mmToLogical*topMargin;
    double xPos;
    dc->SetFont(wxFont(wxFontInfo(10).FaceName(FACENAME)));
    //Vykresleni left header
    if(!m_headerLeftText.IsEmpty())
    {
        textSize = dc->GetTextExtent(m_headerLeftText);
        xPos = leftMargin*mmToLogical;
        dc->DrawText(m_headerLeftText, static_cast<int>(xPos), static_cast<int>(topMarginLogical)-textSize.y);
    }
    //Vykresleni center header
    if(!m_headerCenterText.IsEmpty())
    {
        textSize = dc->GetTextExtent(m_headerCenterText);
        xPos = (((((pageWidthMM - leftMargin - rightMargin)/2.0)+leftMargin)*mmToLogical) - (textSize.x/2.0));
        dc->DrawText(m_headerCenterText, static_cast<int>(xPos), static_cast<int>(topMarginLogical)-textSize.y);
    }
    //Vykresleni right header
    if(!m_headerRightText.IsEmpty())
    {
        textSize = dc->GetTextExtent(m_headerRightText);
        xPos = ((pageWidthMM - rightMargin)*mmToLogical - textSize.x);
        dc->DrawText(m_headerRightText, static_cast<int>(xPos), static_cast<int>(topMarginLogical)-textSize.y);
    }
    //Vykresleni cary
    if(m_headerLine)
    {
        double leftMarginLogical = (mmToLogical*leftMargin);
        double rightMarginLogical = (mmToLogical*(pageWidthMM - rightMargin));
        dc->SetPen(*wxBLACK_PEN);
        dc->DrawLine(static_cast<int>(leftMarginLogical), static_cast<int>(topMarginLogical), static_cast<int>(rightMarginLogical), static_cast<int>(topMarginLogical));
    }
}
