/***************************************************************
 * Name:      elements.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "elements.h"
#include "utils/dxutils.h"
#include "model/dbmodel.h"
#include <wx/tokenzr.h>
#include <wx/arrimpl.cpp>
#include <wx/display.h>

#define SELECTED_RADIUS 1.5

WX_DEFINE_OBJARRAY(dxElementsArray)

Element::Element()
    : m_page(-1), m_selected(false), m_section(ANYSECTION)
{
}

Element::Element(int page)
    : m_page(page), m_selected(false), m_section(ANYSECTION)
{
}

void Element::DrawToDC(wxDC */*dc*/, double /*mmToLogical*/, int /*ydist*/, int /*xdist*/)
{
}

wxXmlNode *Element::createXmlNode() const
{
    wxXmlNode *node = new wxXmlNode(wxXML_ELEMENT_NODE, "Element");
    node->AddAttribute("Type", wxString::Format("%d", ELEMENT));
    node->AddAttribute("Section", wxString::Format("%d", m_section));
    return node;
}

int Element::page() const
{
    return m_page;
}

void Element::setPage(int page)
{
    m_page = page;
}

bool Element::selected() const
{
    return m_selected;
}

void Element::setSelected(bool selected)
{
    m_selected = selected;
}

DOCSECTION Element::section() const
{
    return m_section;
}

void Element::setSection(DOCSECTION section)
{
    m_section = section;
}

LineElement::LineElement()
    : Element(-1), m_start(wxPoint(0,0)), m_end(wxPoint(0,0)), m_style(LineStyle()), m_anchor(false), m_borderTop(false), m_borderBottom(false)
{
}

LineElement::LineElement(int page, wxPoint start, wxPoint end, LineStyle style)
    : Element(page), m_start(start), m_end(end), m_style(style), m_anchor(false), m_borderTop(false), m_borderBottom(false)
{
}

void LineElement::DrawToDC(wxDC *dc, double mmToLogical, int ydist, int xdist)
{
    dc->SetPen(wxPen(m_style.lineColour(), m_style.width(), m_style.style()));
    dc->DrawLine(static_cast<int>((m_start.x+xdist)*mmToLogical), static_cast<int>((m_start.y+ydist)*mmToLogical),
                 static_cast<int>((m_end.x+xdist)*mmToLogical), static_cast<int>((m_end.y+ydist)*mmToLogical));
    if(m_selected)
    {
        dc->SetPen(*wxBLACK_PEN);
        dc->SetBrush(*wxGREEN_BRUSH);
        dc->DrawCircle(static_cast<int>((m_start.x+xdist)*mmToLogical), static_cast<int>((m_start.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_end.x+xdist)*mmToLogical), static_cast<int>((m_end.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
    }
}

wxPoint LineElement::start() const
{
    return m_start;
}

void LineElement::setStart(const wxPoint &start)
{
    m_start = start;
}

wxPoint LineElement::end() const
{
    return m_end;
}

void LineElement::setEnd(const wxPoint &end)
{
    m_end = end;
}

LineStyle LineElement::lineStyle() const
{
    return m_style;
}

void LineElement::setLineStyle(const LineStyle &style)
{
    m_style = style;
}

wxXmlNode *LineElement::createXmlNode() const
{
    wxXmlNode *node = new wxXmlNode(wxXML_ELEMENT_NODE, "Element");
    node->AddAttribute("Type", wxString::Format("%d", LINE));
    node->AddAttribute("Section", wxString::Format("%d", m_section));
    node->AddAttribute("StartX", wxString::Format("%d", m_start.x));
    node->AddAttribute("StartY", wxString::Format("%d", m_start.y));
    node->AddAttribute("EndX", wxString::Format("%d", m_end.x));
    node->AddAttribute("EndY", wxString::Format("%d", m_end.y));
    node->AddAttribute("anchor", wxString::Format("%d", m_anchor));
    node->AddAttribute("borderTop", wxString::Format("%d", m_borderTop));
    node->AddAttribute("borderBottom", wxString::Format("%d", m_borderBottom));
    node->AddAttribute("Page", wxString::Format("%d", m_page));
    node->AddChild(m_style.createXmlNode());
    return node;
}

bool LineElement::fromXmlNode(const wxXmlNode *node)
{
    if(!node) return false;
    if(static_cast<ELEMENTTYPE>(wxAtoi(node->GetAttribute("Type"))) != LINE) return false;
    m_start = wxPoint(wxAtoi(node->GetAttribute("StartX", "0")), wxAtoi(node->GetAttribute("StartY", "0")));
    m_end = wxPoint(wxAtoi(node->GetAttribute("EndX", "0")), wxAtoi(node->GetAttribute("EndY", "0")));
    m_style.fromXmlNode(node->GetChildren());
    m_page = wxAtoi(node->GetAttribute("Page", "1"));
    m_section = static_cast<DOCSECTION>(wxAtoi(node->GetAttribute("Section", "9")));
    m_anchor = wxAtoi(node->GetAttribute("anchor", "0"));
    m_borderTop = wxAtoi(node->GetAttribute("borderTop", "0"));
    m_borderBottom = wxAtoi(node->GetAttribute("borderBottom", "0"));
    return true;
}

bool LineElement::anchor() const
{
    return m_anchor;
}

void LineElement::setAnchor(bool anchor)
{
    m_anchor = anchor;
}

bool LineElement::borderTop() const
{
    return m_borderTop;
}

void LineElement::setBorderTop(bool borderTop)
{
    m_borderTop = borderTop;
}
bool LineElement::borderBottom() const
{
    return m_borderBottom;
}

void LineElement::setBorderBottom(bool borderBottom)
{
    m_borderBottom = borderBottom;
}

int LineElement::height(bool /*inpx*/) const
{
    if(m_anchor || m_borderBottom || m_borderTop)
    {
        return 0;
    }
    return m_end.y - m_start.y;
}

int LineElement::posY() const
{
    if(m_anchor || m_borderBottom || m_borderTop)
    {
        return 0;
    }
    return m_start.y;
}

TextElement::TextElement()
    : Element(-1), m_width(0), m_alignment(wxTEXT_ALIGNMENT_LEFT), m_forceSpace(false)
{
}

TextElement::TextElement(int page, const wxString &value, wxPoint pos, int width, wxTextAttrAlignment alignment)
    : Element(page), m_value(value), m_pos(pos), m_width(width), m_alignment(alignment), m_forceSpace(false)
{
}

TextElement::TextElement(int page, const wxString &value, wxPoint pos, int width, wxTextAttrAlignment alignment, TextStyle style)
    : Element(page), m_value(value), m_pos(pos), m_width(width), m_alignment(alignment), m_style(style), m_forceSpace(false)
{
}

void TextElement::DrawToDC(wxDC *dc, double mmToLogical, int ydist, int xdist)
{
    dc->SetFont(m_style.font());
    dc->SetTextForeground(m_style.fontColour());
    dc->SetTextBackground(m_style.backColour());
    int textW, textH;
    textH = height();
    if(m_style.backColour() != *wxWHITE)
    {
        dc->SetPen(*wxTRANSPARENT_PEN);
        dc->SetBrush(wxBrush(m_style.backColour()));
        dc->DrawRectangle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>(m_width*mmToLogical), static_cast<int>(textH*mmToLogical));
    }
    wxString wrappedLine, line;
    wxArrayString lines = wxStringTokenize(m_value, wxT("\n") );
    wxArrayString wrappedLines;
    for(size_t l = 0; l < lines.GetCount(); l++)
    {
        line = lines[l];
        wrappedLine = "";
        size_t space = 0;
        for(size_t w = 0; w < line.Len(); w++)
        {
            wrappedLine += line[w];
            if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
            if(m_forceSpace) space = w;
            dc->GetTextExtent(wrappedLine, &textW, &textH);
            if(textW > 0.9*m_width*mmToLogical && space)
            {
                wrappedLines.Add(wrappedLine.Left(wrappedLine.Len()-w+space));
                wrappedLine = wxT("");
                w = space;
                space = 0;
            }
        }
        if(wrappedLine != wxEmptyString) wrappedLines.Add(wrappedLine);
    }
    double y = (m_pos.y+ydist)*1.0;
    for(size_t l = 0; l < wrappedLines.GetCount(); l++)
    {
        dc->GetTextExtent(wrappedLines[l], &textW, &textH);
        double cos = 0; // cell off-set
        switch(m_alignment)
        {
        case wxTEXT_ALIGNMENT_LEFT:
            cos = CELL_TEXT_OFFSET*mmToLogical;
            break;
        case wxTEXT_ALIGNMENT_RIGHT:
            cos = m_width*mmToLogical - textW - CELL_TEXT_OFFSET*mmToLogical;
            break;
        default:
            cos = (m_width*mmToLogical - textW) / 2;
        }
        dc->DrawText(wrappedLines[l], static_cast<int>((m_pos.x+xdist)*mmToLogical)+wxRound(cos), wxRound(y*mmToLogical));
        y += textH/mmToLogical;
    }
    if(m_selected)
    {
        textH = height();
        dc->SetPen(*wxBLACK_PEN);
        dc->SetBrush(*wxGREEN_BRUSH);
        dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+m_width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+textH)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+m_width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+textH)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->SetPen(wxPen(*wxBLACK, 1, wxPENSTYLE_SHORT_DASH));
        dc->SetBrush(*wxTRANSPARENT_BRUSH);
        dc->DrawRectangle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>(m_width*mmToLogical), static_cast<int>(textH*mmToLogical));
    }
}

wxString TextElement::value() const
{
    return m_value;
}

void TextElement::setValue(const wxString &value)
{
    m_value = value;
}

wxPoint TextElement::pos() const
{
    return m_pos;
}

void TextElement::setPos(const wxPoint &pos)
{
    m_pos = pos;
}

int TextElement::width() const
{
    return m_width;
}

void TextElement::setWidth(int width)
{
    m_width = width;
}

wxTextAttrAlignment TextElement::alignment() const
{
    return m_alignment;
}

void TextElement::setAlignment(const wxTextAttrAlignment &alignment)
{
    m_alignment = alignment;
}

TextStyle TextElement::style() const
{
    return m_style;
}

void TextElement::setStyle(const TextStyle &style)
{
    m_style = style;
}

wxString TextElement::fontString() const
{
    return wxToString(m_style.font());
}

int TextElement::height(bool inpx) const
{
    double lineHeight = 0.0;
    if(inpx) lineHeight = 1.2 * m_style.font().GetPixelSize().y;
    else lineHeight = 1.2 * m_style.font().GetPointSize() * (25.4/72.);
    int lnum = 1;
    wxScreenDC dc;
    double cw = dxutils::mmToPx(0.9*m_width, &dc, true);
    dc.SetFont(m_style.font());
    int textW, textH, lw;
    wxString wrappedLine, line;
    wxArrayString lines = wxStringTokenize(m_value, wxT("\n"));
    lw = static_cast<int>(lines.GetCount());
    for(size_t l = 0; l < lines.GetCount(); ++l)
    {
        line = lines[l];
        wrappedLine = wxT("");
        size_t space = 0;
        for(size_t w = 0; w < line.Len(); ++w)
        {
            wrappedLine += line[w];
            if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
            if(m_forceSpace) space = w;
            dc.GetTextExtent(wrappedLine, &textW, &textH);
            if(textW > cw && space)
            {
                wrappedLine = wxT("");
                w = space;
                space = 0;
                lw++;
            }
        }
    }
    if(lw > lnum)
        lnum = lw;
    return wxRound(lnum*lineHeight);
}

wxXmlNode *TextElement::createXmlNode() const
{
    wxXmlNode *node = new wxXmlNode(wxXML_ELEMENT_NODE, "Element");
    node->AddAttribute("Type", wxString::Format("%d", TEXT));
    node->AddAttribute("Section", wxString::Format("%d", m_section));
    node->AddAttribute("Value", m_value);
    node->AddAttribute("X", wxString::Format("%d", m_pos.x));
    node->AddAttribute("Y", wxString::Format("%d", m_pos.y));
    node->AddAttribute("Width", wxString::Format("%d", m_width));
    node->AddAttribute("Alignment", wxString::Format("%d", m_alignment));
    node->AddAttribute("Page", wxString::Format("%d", m_page));
    node->AddAttribute("ForceSpace", wxString::Format("%d", m_forceSpace));
    node->AddChild(m_style.createXmlNode());
    return node;
}

bool TextElement::fromXmlNode(const wxXmlNode *node)
{
    if(!node) return false;
    if(static_cast<ELEMENTTYPE>(wxAtoi(node->GetAttribute("Type"))) != TEXT) return false;
    m_value = node->GetAttribute("Value");
    m_pos = wxPoint(wxAtoi(node->GetAttribute("X", "0")), wxAtoi(node->GetAttribute("Y", "0")));
    m_width = wxAtoi(node->GetAttribute("Width", "0"));
    m_alignment = static_cast<wxTextAttrAlignment>(wxAtoi(node->GetAttribute("Alignment", "1")));
    m_section = static_cast<DOCSECTION>(wxAtoi(node->GetAttribute("Section", "9")));
    m_style.fromXmlNode(node->GetChildren());
    m_page = wxAtoi(node->GetAttribute("Page", "1"));
    m_forceSpace = wxAtoi(node->GetAttribute("ForceSpace", "0"));
    return true;
}

int TextElement::posY() const
{
    return m_pos.y;
}

bool TextElement::forceSpace() const
{
    return m_forceSpace;
}

void TextElement::setForceSpace(bool forceSpace)
{
    m_forceSpace = forceSpace;
}

TextParagraphElement::TextParagraphElement()
    : Element(-1), m_width(0), m_alignment(wxTEXT_ALIGNMENT_LEFT), m_border(false)
{
}

TextParagraphElement::TextParagraphElement(int page, const wxArrayString &value, wxPoint pos, int width, wxTextAttrAlignment alignment)
    : Element(page), m_value(value), m_pos(pos), m_width(width), m_alignment(alignment), m_border(false), m_borderStyle(LineStyle())
{
}

TextParagraphElement::TextParagraphElement(int page, const wxArrayString &value, wxPoint pos, int width, wxTextAttrAlignment alignment, TextStyle style)
    : Element(page), m_value(value), m_pos(pos), m_width(width), m_alignment(alignment), m_style(style), m_border(false), m_borderStyle(LineStyle())
{
}

void TextParagraphElement::DrawToDC(wxDC *dc, double mmToLogical, int ydist, int xdist)
{
    dc->SetFont(m_style.font());
    int textW, textH;
    textH = height();
    if(m_style.backColour() != *wxWHITE)
    {
        dc->SetPen(*wxTRANSPARENT_PEN);
        dc->SetBrush(wxBrush(m_style.backColour()));
        dc->DrawRectangle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>(m_width*mmToLogical), static_cast<int>(textH*mmToLogical));
    }
    if(m_border)
    {
        dc->SetPen(wxPen(m_borderStyle.lineColour(), m_borderStyle.width(), m_borderStyle.style()));
        dc->SetBrush(*wxTRANSPARENT_BRUSH);
        dc->DrawRectangle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>(m_width*mmToLogical), static_cast<int>(textH*mmToLogical));
    }
    dc->SetTextForeground(m_style.fontColour());
    dc->SetTextBackground(m_style.backColour());
    wxString wrappedLine, line;
    wxArrayString lines;
    for(size_t i=0; i<m_value.GetCount(); i++)
    {
        wxArrayString tok = wxStringTokenize(m_value[i], wxT("\n"));
        for(size_t j=0; j<tok.GetCount(); j++)
        {
            lines.Add(tok[j]);
        }
    }
    wxArrayString wrappedLines;
    for(size_t l = 0; l < lines.GetCount(); l++)
    {
        line = lines[l];
        wrappedLine = "";
        size_t space = 0;
        for(size_t w = 0; w < line.Len(); w++)
        {
            wrappedLine += line[w];
            if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
            dc->GetTextExtent(wrappedLine, &textW, &textH);
            if(textW > 0.9*m_width*mmToLogical && space)
            {
                wrappedLines.Add(wrappedLine.Left(wrappedLine.Len()-w+space));
                wrappedLine = wxT("");
                w = space;
                space = 0;
            }
        }
        if(wrappedLine != wxEmptyString) wrappedLines.Add(wrappedLine);
    }
    double y = (m_pos.y+ydist)*1.0;
    for(size_t l = 0; l < wrappedLines.GetCount(); l++)
    {
        dc->GetTextExtent(wrappedLines[l], &textW, &textH);
        double cos = 0; // cell off-set
        switch(m_alignment)
        {
        case wxTEXT_ALIGNMENT_LEFT:
            cos = CELL_TEXT_OFFSET*mmToLogical;
            break;
        case wxTEXT_ALIGNMENT_RIGHT:
            cos = m_width*mmToLogical - textW - CELL_TEXT_OFFSET*mmToLogical;
            break;
        default:
            cos = (m_width*mmToLogical - textW) / 2;
        }
        dc->DrawText(wrappedLines[l], static_cast<int>((m_pos.x+xdist)*mmToLogical)+wxRound(cos), wxRound(y*mmToLogical));
        y += textH/mmToLogical;
    }
    if(m_selected)
    {
        textH = height();
        dc->SetPen(*wxBLACK_PEN);
        dc->SetBrush(*wxGREEN_BRUSH);
        dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+m_width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+textH)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+m_width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+textH)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->SetPen(wxPen(*wxBLACK, 1, wxPENSTYLE_SHORT_DASH));
        dc->SetBrush(*wxTRANSPARENT_BRUSH);
        dc->DrawRectangle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>(m_width*mmToLogical), static_cast<int>(textH*mmToLogical));
    }
}

wxArrayString TextParagraphElement::value() const
{
    return m_value;
}

void TextParagraphElement::setValue(const wxArrayString &value)
{
    m_value = value;
}

wxPoint TextParagraphElement::pos() const
{
    return m_pos;
}

void TextParagraphElement::setPos(const wxPoint &pos)
{
    m_pos = pos;
}

int TextParagraphElement::width() const
{
    return m_width;
}

void TextParagraphElement::setWidth(int width)
{
    m_width = width;
}

wxTextAttrAlignment TextParagraphElement::alignment() const
{
    return m_alignment;
}

void TextParagraphElement::setAlignment(const wxTextAttrAlignment &alignment)
{
    m_alignment = alignment;
}

TextStyle TextParagraphElement::style() const
{
    return m_style;
}

void TextParagraphElement::setStyle(const TextStyle &style)
{
    m_style = style;
}

wxString TextParagraphElement::fontString() const
{
    return wxToString(m_style.font());
}

int TextParagraphElement::height(bool inpx) const
{
    double lineHeight = 0.0;
    if(inpx) lineHeight = 1.2 * m_style.font().GetPixelSize().y;
    else lineHeight = 1.2 * m_style.font().GetPointSize() * (25.4/72.);
    int lnum = 1;
    wxScreenDC dc;
    double cw = dxutils::mmToPx(0.9*m_width, &dc, true);
    dc.SetFont(m_style.font());
    int textW, textH, lw;
    wxString wrappedLine, line;
    wxArrayString lines;
    for(size_t i=0; i<m_value.GetCount(); i++)
    {
        wxArrayString tok = wxStringTokenize(m_value[i], wxT("\n"));
        for(size_t j=0; j<tok.GetCount(); j++)
        {
            lines.Add(tok[j]);
        }
    }
    lw = static_cast<int>(lines.GetCount());
    for(size_t l = 0; l < lines.GetCount(); ++l)
    {
        line = lines[l];
        wrappedLine = wxT("");
        size_t space = 0;
        for(size_t w = 0; w < line.Len(); ++w)
        {
            wrappedLine += line[w];
            if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
            dc.GetTextExtent(wrappedLine, &textW, &textH);
            if(textW > cw && space)
            {
                wrappedLine = wxT("");
                w = space;
                space = 0;
                lw++;
            }
        }
    }
    if(lw > lnum)
        lnum = lw;
    return wxRound(lnum*lineHeight);
}

bool TextParagraphElement::border() const
{
    return m_border;
}

void TextParagraphElement::setBorder(bool border)
{
    m_border = border;
}

LineStyle TextParagraphElement::borderStyle() const
{
    return m_borderStyle;
}

void TextParagraphElement::setBorderStyle(const LineStyle &borderStyle)
{
    m_borderStyle = borderStyle;
}

wxXmlNode *TextParagraphElement::createXmlNode() const
{
    wxXmlNode *node = new wxXmlNode(wxXML_ELEMENT_NODE, "Element");
    node->AddAttribute("Type", wxString::Format("%d", TEXTPARAGRAPH));
    node->AddAttribute("Section", wxString::Format("%d", m_section));
    node->AddAttribute("X", wxString::Format("%d", m_pos.x));
    node->AddAttribute("Y", wxString::Format("%d", m_pos.y));
    node->AddAttribute("Width", wxString::Format("%d", m_width));
    node->AddAttribute("Alignment", wxString::Format("%d", m_alignment));
    node->AddAttribute("Border", wxString::Format("%d", m_border));
    node->AddAttribute("Page", wxString::Format("%d", m_page));
    node->AddChild(m_style.createXmlNode());
    node->AddChild(m_borderStyle.createXmlNode());
    wxXmlNode *valuesNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Values");
    for(size_t i=0; i < m_value.GetCount(); i++)
    {
        wxXmlNode *valueNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Value");
        valueNode->AddChild(new wxXmlNode(wxXML_TEXT_NODE, "", m_value[i]));
        valuesNode->AddChild(valueNode);
    }
    node->AddChild(valuesNode);
    return node;
}

bool TextParagraphElement::fromXmlNode(const wxXmlNode *node)
{
    if(!node) return false;
    if(static_cast<ELEMENTTYPE>(wxAtoi(node->GetAttribute("Type"))) != TEXTPARAGRAPH) return false;
    m_pos = wxPoint(wxAtoi(node->GetAttribute("X", "0")), wxAtoi(node->GetAttribute("Y", "0")));
    m_width = wxAtoi(node->GetAttribute("Width", "0"));
    m_alignment = static_cast<wxTextAttrAlignment>(wxAtoi(node->GetAttribute("Alignment", "1")));
    m_section = static_cast<DOCSECTION>(wxAtoi(node->GetAttribute("Section", "9")));
    m_border = wxAtoi(node->GetAttribute("Border", "0"));
    m_page = wxAtoi(node->GetAttribute("Page", "1"));
    wxXmlNode *styleNode = node->GetChildren();
    m_style.fromXmlNode(styleNode);
    wxXmlNode *borderNode = styleNode->GetNext();
    m_borderStyle.fromXmlNode(borderNode);
    wxXmlNode *valuesNode = borderNode->GetNext();
    wxXmlNode *valueNode = valuesNode->GetChildren();
    while(valueNode)
    {
        m_value.Add(valueNode->GetNodeContent());
        valueNode = valueNode->GetNext();
    }
    return true;
}

int TextParagraphElement::posY() const
{
    return m_pos.y;
}

TableRowElement::TableRowElement()
    : Element(-1), m_border(false)
{
}

TableRowElement::TableRowElement(int page, const wxArrayString &value, wxPoint pos, wxArrayInt width)
    : Element(page), m_value(value), m_pos(pos), m_width(width), m_border(false), m_borderStyle(LineStyle())
{
    for(size_t i=0; i < m_value.GetCount(); i++)
        m_alignment.Add(wxTEXT_ALIGNMENT_LEFT);
}

TableRowElement::TableRowElement(int page, const wxArrayString &value, wxPoint pos, wxArrayInt width, wxArrayInt alignment)
    : Element(page), m_value(value), m_pos(pos), m_width(width), m_alignment(alignment), m_border(false), m_borderStyle(LineStyle())
{
}

TableRowElement::TableRowElement(int page, const wxArrayString &value, wxPoint pos, wxArrayInt width, wxArrayInt alignment, TextStyle style)
    : Element(page), m_value(value), m_pos(pos), m_width(width), m_alignment(alignment), m_style(style), m_border(false), m_borderStyle(LineStyle())
{
}

void TableRowElement::DrawToDC(wxDC *dc, double mmToLogical, int ydist, int xdist)
{
    if((m_value.GetCount() != m_width.GetCount()) && (m_value.GetCount() != m_alignment.GetCount()))
    {
        return;
    }
    int eheight = height();
    int width = totalWidth();
    if(style().backColour() != *wxWHITE)
    {
        dc->SetPen(*wxTRANSPARENT_PEN);
        dc->SetBrush(wxBrush(m_style.backColour()));
        dc->DrawRectangle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>(width*mmToLogical), static_cast<int>(eheight*mmToLogical));
    }
    if(m_border)
    {
        dc->SetPen(wxPen(m_borderStyle.lineColour(), m_borderStyle.width(), m_borderStyle.style()));
        dc->SetBrush(*wxTRANSPARENT_BRUSH);
        dc->DrawLine(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>((m_pos.x+width+xdist)*mmToLogical),
                     static_cast<int>((m_pos.y+ydist)*mmToLogical));
        dc->DrawLine(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+eheight)*mmToLogical),
                     static_cast<int>((m_pos.x+width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+eheight)*mmToLogical));
        width = 0;
        for(size_t i=0; i<m_width.GetCount(); i++)
        {
            dc->DrawLine(static_cast<int>((m_pos.x+width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>((m_pos.x+width+xdist)*mmToLogical),
                         static_cast<int>((m_pos.y+ydist+eheight)*mmToLogical));
            width += m_width[i];
        }
        dc->DrawLine(static_cast<int>((m_pos.x+width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>((m_pos.x+width+xdist)*mmToLogical),
                     static_cast<int>((m_pos.y+ydist+eheight)*mmToLogical));
    }
    dc->SetFont(m_style.font());
    dc->SetTextForeground(m_style.fontColour());
    dc->SetTextBackground(m_style.backColour());
    for(size_t i=0; i<m_value.GetCount(); i++)
    {
        int textW, textH;
        wxString wrappedLine, line;
        wxArrayString lines = wxStringTokenize(m_value[i], wxT("\n"));
        wxArrayString wrappedLines;
        for(size_t l = 0; l < lines.GetCount(); l++)
        {
            line = lines[l];
            wrappedLine = "";
            size_t space = 0;
            for(size_t w = 0; w < line.Len(); w++)
            {
                wrappedLine += line[w];
                if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
                dc->GetTextExtent(wrappedLine, &textW, &textH);
                if(textW > 0.9*m_width[i]*mmToLogical && space)
                {
                    wrappedLines.Add(wrappedLine.Left(wrappedLine.Len()-w+space));
                    wrappedLine = wxT("");
                    w = space;
                    space = 0;
                }
            }
            if(wrappedLine != wxEmptyString) wrappedLines.Add(wrappedLine);
        }
        double x = m_pos.x+xdist;
        if(i)
        {
            for(size_t w = 0; w <= (i-1); w++)
            {
                x += m_width[w];
            }
        }
        double y = (m_pos.y+ydist)*1.0;
        for(size_t l = 0; l < wrappedLines.GetCount(); l++)
        {
            dc->GetTextExtent(wrappedLines[l], &textW, &textH);
            double cos = 0; // cell off-set
            switch(m_alignment[i])
            {
            case wxTEXT_ALIGNMENT_LEFT:
                cos = CELL_TEXT_OFFSET*mmToLogical;
                break;
            case wxTEXT_ALIGNMENT_RIGHT:
                cos = m_width[i]*mmToLogical - textW - CELL_TEXT_OFFSET*mmToLogical;
                break;
            default:
                cos = (m_width[i]*mmToLogical - textW) / 2;
            }
            dc->DrawText(wrappedLines[l], static_cast<int>(x*mmToLogical)+wxRound(cos), wxRound(y*mmToLogical));
            y += textH/mmToLogical;
        }
    }
    if(m_selected)
    {
        eheight = height();
        width = totalWidth();
        dc->SetPen(*wxBLACK_PEN);
        dc->SetBrush(*wxGREEN_BRUSH);
        dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+eheight)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+eheight)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->SetPen(wxPen(*wxBLACK, 1, wxPENSTYLE_SHORT_DASH));
        dc->SetBrush(*wxTRANSPARENT_BRUSH);
        dc->DrawRectangle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>(width*mmToLogical), static_cast<int>(eheight*mmToLogical));
        width = 0;
        for(size_t i=0; i<m_width.GetCount(); ++i)
        {
            dc->DrawLine(static_cast<int>((m_pos.x+width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>((m_pos.x+width+xdist)*mmToLogical),
                         static_cast<int>((m_pos.y+ydist+eheight)*mmToLogical));
            width += m_width[i];
        }
    }
}

wxArrayString TableRowElement::value() const
{
    return m_value;
}

void TableRowElement::setValue(const wxArrayString &value)
{
    m_value = value;
}

wxPoint TableRowElement::pos() const
{
    return m_pos;
}

void TableRowElement::setPos(const wxPoint &pos)
{
    m_pos = pos;
}

wxArrayInt TableRowElement::width() const
{
    return m_width;
}

void TableRowElement::setWidth(const wxArrayInt &width)
{
    m_width = width;
}

wxArrayInt TableRowElement::alignment() const
{
    return m_alignment;
}

void TableRowElement::setAlignment(const wxArrayInt &alignment)
{
    m_alignment = alignment;
}

TextStyle TableRowElement::style() const
{
    return m_style;
}

void TableRowElement::setStyle(const TextStyle &style)
{
    m_style = style;
}

wxString TableRowElement::fontString() const
{
    return wxToString(m_style.font());
}

int TableRowElement::height(bool inpx) const
{
    if((m_value.GetCount() != m_width.GetCount()) && (m_value.GetCount() != m_alignment.GetCount()))
    {
        return 0;
    }
    double lineHeight = 0.0;
    if(inpx) lineHeight = 1.2 * m_style.font().GetPixelSize().y;
    else lineHeight = 1.2 * m_style.font().GetPointSize() * (25.4/72.);
    int lnum = 1;
    wxScreenDC dc;
    for(size_t i=0; i<m_value.GetCount(); i++)
    {
        double cw = dxutils::mmToPx(0.9*m_width[i], &dc, true);
        dc.SetFont(m_style.font());
        int textW, textH, lw;
        wxString wrappedLine, line;
        wxArrayString lines = wxStringTokenize(m_value[i], wxT("\n"));
        lw = static_cast<int>(lines.GetCount());
        for(size_t l = 0; l < lines.GetCount(); ++l)
        {
            line = lines[l];
            bool someSpace = dxutils::pocetZnaku(line, ' ')+dxutils::pocetZnaku(line, '\t');
            wrappedLine = wxT("");
            size_t space = 0;
            for(size_t w = 0; w < line.Len(); ++w)
            {
                wrappedLine += line[w];
                if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
                if(!someSpace) space = w;
                dc.GetTextExtent(wrappedLine, &textW, &textH);
                if(textW > cw && space)
                {
                    wrappedLine = wxT("");
                    w = space;
                    space = 0;
                    lw++;
                }
            }
        }
        if(lw > lnum)
            lnum = lw;
    }
    return wxRound(lnum*lineHeight);
}

bool TableRowElement::border() const
{
    return m_border;
}

void TableRowElement::setBorder(bool border)
{
    m_border = border;
}

LineStyle TableRowElement::borderStyle() const
{
    return m_borderStyle;
}

void TableRowElement::setBorderStyle(const LineStyle &borderStyle)
{
    m_borderStyle = borderStyle;
}

int TableRowElement::totalWidth()
{
    int width = 0;
    for(size_t i=0; i<m_width.GetCount(); i++)
    {
        width += m_width[i];
    }
    return width;
}

wxXmlNode *TableRowElement::createXmlNode() const
{
    wxXmlNode *node = new wxXmlNode(wxXML_ELEMENT_NODE, "Element");
    node->AddAttribute("Type", wxString::Format("%d", TABLEROW));
    node->AddAttribute("Section", wxString::Format("%d", m_section));
    node->AddAttribute("X", wxString::Format("%d", m_pos.x));
    node->AddAttribute("Y", wxString::Format("%d", m_pos.y));
    node->AddAttribute("Border", wxString::Format("%d", m_border));
    node->AddAttribute("Page", wxString::Format("%d", m_page));
    node->AddChild(m_style.createXmlNode());
    node->AddChild(m_borderStyle.createXmlNode());
    wxXmlNode *valuesNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Values");
    for(size_t i=0; i < m_value.GetCount(); i++)
    {
        wxXmlNode *valueNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Value");
        valueNode->AddChild(new wxXmlNode(wxXML_TEXT_NODE, "", m_value[i]));
        valuesNode->AddChild(valueNode);
    }
    node->AddChild(valuesNode);
    wxXmlNode *widthsNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Widths");
    for(size_t i=0; i < m_width.GetCount(); i++)
    {
        wxXmlNode *widthNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Width");
        widthNode->AddChild(new wxXmlNode(wxXML_TEXT_NODE, "", wxString::Format("%d", m_width[i])));
        widthsNode->AddChild(widthNode);
    }
    node->AddChild(widthsNode);
    wxXmlNode *alignmentsNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Alignments");
    for(size_t i=0; i < m_alignment.GetCount(); i++)
    {
        wxXmlNode *alignmentNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Alignment");
        alignmentNode->AddChild(new wxXmlNode(wxXML_TEXT_NODE, "", wxString::Format("%d", m_alignment[i])));
        alignmentsNode->AddChild(alignmentNode);
    }
    node->AddChild(alignmentsNode);
    return node;
}

bool TableRowElement::fromXmlNode(const wxXmlNode *node)
{
    if(!node) return false;
    if(static_cast<ELEMENTTYPE>(wxAtoi(node->GetAttribute("Type"))) != TABLEROW) return false;
    m_pos = wxPoint(wxAtoi(node->GetAttribute("X", "0")), wxAtoi(node->GetAttribute("Y", "0")));
    m_border = wxAtoi(node->GetAttribute("Border", "0"));
    m_page = wxAtoi(node->GetAttribute("Page", "1"));
    m_section = static_cast<DOCSECTION>(wxAtoi(node->GetAttribute("Section", "9")));
    wxXmlNode *styleNode = node->GetChildren();
    m_style.fromXmlNode(styleNode);
    wxXmlNode *borderNode = styleNode->GetNext();
    m_borderStyle.fromXmlNode(borderNode);
    wxXmlNode *valuesNode = borderNode->GetNext();
    wxXmlNode *valueNode = valuesNode->GetChildren();
    while(valueNode)
    {
        m_value.Add(valueNode->GetNodeContent());
        valueNode = valueNode->GetNext();
    }
    wxXmlNode *widthsNode = valuesNode->GetNext();
    wxXmlNode *widthNode = widthsNode->GetChildren();
    while(widthNode)
    {
        m_width.Add(wxAtoi(widthNode->GetNodeContent()));
        widthNode = widthNode->GetNext();
    }
    wxXmlNode *alignmentsNode = widthsNode->GetNext();
    wxXmlNode *alignmentNode = alignmentsNode->GetChildren();
    while(alignmentNode)
    {
        m_alignment.Add(static_cast<wxTextAttrAlignment>(wxAtoi(alignmentNode->GetNodeContent())));
        alignmentNode = alignmentNode->GetNext();
    }
    return true;
}

wxString TableRowElement::cellValue(size_t index) const
{
    if(index >= m_value.GetCount()) return wxEmptyString;
    return m_value[index];
}

void TableRowElement::setCellValue(size_t index, const wxString& value)
{
    if(index >= m_value.GetCount()) return;
    m_value[index] = value;
}

void TableRowElement::addCellValue(const wxString& value)
{
    m_value.Add(value);
}

int TableRowElement::cellWidth(size_t index) const
{
    if(index >= m_width.GetCount()) return 0;
    return m_width[index];
}

void TableRowElement::setCellWidth(size_t index, int width)
{
    if(index >= m_width.GetCount()) return;
    m_width[index] = width;
}

void TableRowElement::addCellWidth(int width)
{
    m_width.Add(width);
}

wxTextAttrAlignment TableRowElement::cellAlignment(size_t index) const
{
    if(index >= m_alignment.GetCount()) return wxTEXT_ALIGNMENT_LEFT;
    return static_cast<wxTextAttrAlignment>(m_alignment[index]);
}

void TableRowElement::setCellAlignment(size_t index, wxTextAttrAlignment alignment)
{
    if(index >= m_alignment.GetCount()) return;
    m_alignment[index] = alignment;
}

void TableRowElement::addCellAlignment(wxTextAttrAlignment alignment)
{
    m_alignment.Add(alignment);
}

void TableRowElement::deleteCell(size_t index)
{
    if(index >= m_value.GetCount()) return;
    if((m_value.GetCount() != m_width.GetCount()) && (m_value.GetCount() != m_alignment.GetCount())) return;
    m_value.RemoveAt(index);
    m_width.RemoveAt(index);
    m_alignment.RemoveAt(index);
}

int TableRowElement::posY() const
{
    return m_pos.y;
}

ImageElement::ImageElement()
    : Element(-1), m_width(0), m_alignment(wxTEXT_ALIGNMENT_LEFT), m_isqrcode(false)
{
}

ImageElement::ImageElement(int page, const wxString &path, wxPoint pos, int width, wxTextAttrAlignment alignment)
    : Element(page), m_path(path), m_pos(pos), m_width(width), m_alignment(alignment), m_isqrcode(false)
{
}

ImageElement::ImageElement(int page, const wxString &code, wxPoint pos)
    : Element(page), m_path("{qr-platba}"), m_pos(pos), m_alignment(wxTEXT_ALIGNMENT_LEFT), m_qrcodetxt(code), m_isqrcode(true)
{
}

void ImageElement::DrawToDC(wxDC *dc, double mmToLogical, int ydist, int xdist)
{
    if(m_isqrcode)
    {
        QrCode qr = QrCode::encodeText(m_qrcodetxt.mb_str(),QrCode::Ecc::MEDIUM);
        int border = static_cast<int>(QR_BORDER*mmToLogical);
        int pixel = static_cast<int>(QR_PIXEL*mmToLogical);
        m_width = 2*border+qr.getSize()*pixel;
        dc->SetFont(wxFont(wxFontInfo(9).FaceName(FACENAME)));
        int textW, textH;
        dc->GetTextExtent("QR Platba", &textW, &textH);
        dc->SetPen(*wxBLACK_PEN);
        dc->SetBrush(*wxTRANSPARENT_BRUSH);
        dc->DrawLine(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical),
                     static_cast<int>((m_pos.x+xdist)*mmToLogical)+m_width, static_cast<int>((m_pos.y+ydist)*mmToLogical));
        dc->DrawLine(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical),
                     static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical)+m_width);
        dc->DrawLine(static_cast<int>((m_pos.x+xdist)*mmToLogical)+m_width, static_cast<int>((m_pos.y+ydist)*mmToLogical),
                     static_cast<int>((m_pos.x+xdist)*mmToLogical)+m_width, static_cast<int>((m_pos.y+ydist)*mmToLogical)+m_width);
        dc->DrawLine(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical)+m_width,
                     static_cast<int>((m_pos.x+xdist)*mmToLogical)+border-static_cast<int>(QR_MEZERA*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical)+m_width);
        dc->DrawLine(static_cast<int>((m_pos.x+xdist)*mmToLogical)+border+static_cast<int>(QR_MEZERA*mmToLogical)+textW, static_cast<int>((m_pos.y+ydist)*mmToLogical)+m_width,
                     static_cast<int>((m_pos.x+xdist)*mmToLogical)+m_width, static_cast<int>((m_pos.y+ydist)*mmToLogical)+m_width);
        dc->DrawText("QR Platba", static_cast<int>((m_pos.x+xdist)*mmToLogical)+border, static_cast<int>((m_pos.y+ydist)*mmToLogical)+m_width);
        dc->SetPen(*wxTRANSPARENT_PEN);
        dc->SetBrush(*wxBLACK_BRUSH);
        for(int y=0; y<qr.getSize(); y++)
        {
            for(int x=0; x<qr.getSize(); x++)
            {
                const int color=qr.getModule(x, y);  // 0 for white, 1 for black
                if(0!=color)
                {
                    dc->DrawRectangle(static_cast<int>((m_pos.x+xdist)*mmToLogical)+border+pixel*x, static_cast<int>((m_pos.y+ydist)*mmToLogical)+border+pixel*y, pixel, pixel);
                }
            }
        }
        if(m_selected)
        {
            int imageH = height();
            dc->SetPen(*wxBLACK_PEN);
            dc->SetBrush(*wxGREEN_BRUSH);
            dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
            dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical)+m_width, static_cast<int>((m_pos.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
            dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+imageH)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
            dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical)+m_width, static_cast<int>((m_pos.y+ydist+imageH)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
            dc->SetPen(wxPen(*wxBLACK, 1, wxPENSTYLE_SHORT_DASH));
            dc->SetBrush(*wxTRANSPARENT_BRUSH);
            dc->DrawRectangle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), m_width, static_cast<int>(imageH*mmToLogical));
        }
        return;
    }
    else
    {
        wxString path = m_path;
        if(path == "{firma-logo}")
        {
            path = idbmodel->firmaLogo();
        }
        if(path == "{firma-razitko}")
        {
            path = idbmodel->firmaRazitko();
        }
        if(!wxFileExists(path))
        {
            return;
        }
        wxImage image;
        if(image.LoadFile(path))
        {
            double ios = 0;
            switch(m_alignment)
            {
            case wxTEXT_ALIGNMENT_LEFT:
                ios = CELL_TEXT_OFFSET*mmToLogical;
                break;
            case wxTEXT_ALIGNMENT_RIGHT:
                ios = m_width*mmToLogical - image.GetWidth() - CELL_TEXT_OFFSET*mmToLogical;
                break;
            default:
                ios = (m_width*mmToLogical - image.GetWidth()) / 2;
            }
            dc->DrawBitmap(wxBitmap(image), static_cast<int>((m_pos.x+xdist)*mmToLogical)+wxRound(ios), static_cast<int>((m_pos.y+ydist)*mmToLogical)+wxRound((height()*mmToLogical-image.GetHeight())/2.0));
        }
    }
    if(m_selected)
    {
        int imageH = height();
        dc->SetPen(*wxBLACK_PEN);
        dc->SetBrush(*wxGREEN_BRUSH);
        dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+m_width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+imageH)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_pos.x+m_width+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist+imageH)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->SetPen(wxPen(*wxBLACK, 1, wxPENSTYLE_SHORT_DASH));
        dc->SetBrush(*wxTRANSPARENT_BRUSH);
        dc->DrawRectangle(static_cast<int>((m_pos.x+xdist)*mmToLogical), static_cast<int>((m_pos.y+ydist)*mmToLogical), static_cast<int>(m_width*mmToLogical),
                          static_cast<int>(imageH*mmToLogical));
    }
}

wxString ImageElement::path() const
{
    return m_path;
}

void ImageElement::setPath(const wxString &path)
{
    m_path = path;
}

wxPoint ImageElement::pos() const
{
    return m_pos;
}

void ImageElement::setPos(const wxPoint &pos)
{
    m_pos = pos;
}

int ImageElement::width() const
{
    return m_width;
}

void ImageElement::setWidth(int width)
{
    m_width = width;
}

wxTextAttrAlignment ImageElement::alignment() const
{
    return m_alignment;
}

void ImageElement::setAlignment(const wxTextAttrAlignment &alignment)
{
    m_alignment = alignment;
}

int ImageElement::height(bool /*inpx*/) const
{
    if(m_isqrcode)
    {
        return QrCode::encodeText(m_qrcodetxt.mb_str(),QrCode::Ecc::MEDIUM).getSize()*QR_PIXEL+2*QR_BORDER+
                wxRound(1.2 * wxFont(wxFontInfo(9).FaceName(FACENAME)).GetPointSize() * (25.4/72.));
    }
    wxString path = m_path;
    if(path == "{firma-logo}")
    {
        path = idbmodel->firmaLogo();
    }
    if(path == "{firma-razitko}")
    {
        path = idbmodel->firmaRazitko();
    }
    if(!wxFileExists(path))
    {
        return 0;
    }
    wxImage image;
    if(image.LoadFile(path))
    {
        return wxRound(image.GetHeight() * (25.4/72.));
    }
    return 0;
}

wxSize ImageElement::imagePxSize() const
{
    if(m_isqrcode)
    {
        QrCode qr = QrCode::encodeText(m_qrcodetxt.mb_str(),QrCode::Ecc::MEDIUM);
        return wxSize(6*QR_BORDER+qr.getSize()*QR_PIXEL*3,6*QR_BORDER+qr.getSize()*QR_PIXEL*3
                      +wxRound(1.2*wxFont(wxFontInfo(9).FaceName(FACENAME)).GetPixelSize().y));
    }
    wxString path = m_path;
    if(path == "{firma-logo}")
    {
        path = idbmodel->firmaLogo();
    }
    if(path == "{firma-razitko}")
    {
        path = idbmodel->firmaRazitko();
    }
    if(!wxFileExists(path))
    {
        return wxSize(0,0);
    }
    wxImage image;
    if(image.LoadFile(path))
    {
        return image.GetSize();
    }
    return wxSize(0,0);
}

wxXmlNode *ImageElement::createXmlNode() const
{
    wxXmlNode *node = new wxXmlNode(wxXML_ELEMENT_NODE, "Element");
    node->AddAttribute("Type", wxString::Format("%d", IMAGE));
    node->AddAttribute("Section", wxString::Format("%d", m_section));
    if(m_isqrcode) node->AddAttribute("Path", "{qr-platba}");
    else node->AddAttribute("Path", m_path);
    node->AddAttribute("X", wxString::Format("%d", m_pos.x));
    node->AddAttribute("Y", wxString::Format("%d", m_pos.y));
    node->AddAttribute("Width", wxString::Format("%d", m_width));
    node->AddAttribute("Alignment", wxString::Format("%d", m_alignment));
    node->AddAttribute("Page", wxString::Format("%d", m_page));
    return node;
}

bool ImageElement::fromXmlNode(const wxXmlNode *node)
{
    if(!node) return false;
    if(static_cast<ELEMENTTYPE>(wxAtoi(node->GetAttribute("Type"))) != IMAGE) return false;
    m_path = node->GetAttribute("Path");
    if(m_path == "{qr-platba}")
    {
        m_isqrcode = true;
    }
    else
    {
        m_isqrcode = false;
    }
    m_pos = wxPoint(wxAtoi(node->GetAttribute("X", "0")), wxAtoi(node->GetAttribute("Y", "0")));
    m_width = wxAtoi(node->GetAttribute("Width", "0"));
    m_alignment = static_cast<wxTextAttrAlignment>(wxAtoi(node->GetAttribute("Alignment", "1")));
    m_page = wxAtoi(node->GetAttribute("Page", "1"));
    m_section = static_cast<DOCSECTION>(wxAtoi(node->GetAttribute("Section", "9")));
    return true;
}

int ImageElement::posY() const
{
    return m_pos.y;
}

wxString ImageElement::qrcodetxt() const
{
    return m_qrcodetxt;
}

void ImageElement::setQrcodetxt(const wxString &qrcodetxt)
{
    m_qrcodetxt = qrcodetxt;
}

bool ImageElement::isqrcode() const
{
    return m_isqrcode;
}

void ImageElement::setIsqrcode(bool isqrcode)
{
    m_isqrcode = isqrcode;
}

RectangleElement::RectangleElement()
    : Element(-1), m_start(wxPoint(0,0)), m_size(wxSize(0,0)), m_style(LineStyle()), m_anchor(false), m_fillet(0)
{
}

RectangleElement::RectangleElement(int page, wxPoint start, wxSize size, LineStyle style, int fillet)
    : Element(page), m_start(start), m_size(size), m_style(style), m_anchor(false), m_fillet(fillet)
{
}

void RectangleElement::DrawToDC(wxDC *dc, double mmToLogical, int ydist, int xdist)
{
    dc->SetPen(wxPen(m_style.lineColour(), m_style.width(), m_style.style()));
    dc->SetBrush(*wxTRANSPARENT_BRUSH);
    if(m_fillet) dc->DrawRoundedRectangle(static_cast<int>((m_start.x+xdist)*mmToLogical),static_cast<int>((m_start.y+ydist)*mmToLogical),
                                          static_cast<int>(m_size.x*mmToLogical), static_cast<int>(m_size.y*mmToLogical), static_cast<int>(m_fillet*mmToLogical));
    else dc->DrawRectangle(static_cast<int>((m_start.x+xdist)*mmToLogical), static_cast<int>((m_start.y+ydist)*mmToLogical),
                           static_cast<int>(m_size.x*mmToLogical), static_cast<int>(m_size.y*mmToLogical));
    if(m_selected)
    {
        dc->SetPen(*wxBLACK_PEN);
        dc->SetBrush(*wxGREEN_BRUSH);
        dc->DrawCircle(static_cast<int>((m_start.x+xdist)*mmToLogical), static_cast<int>((m_start.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_start.x+xdist)*mmToLogical), static_cast<int>((m_start.y+m_size.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_start.x+m_size.x+xdist)*mmToLogical), static_cast<int>((m_start.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
        dc->DrawCircle(static_cast<int>((m_start.x+m_size.x+xdist)*mmToLogical), static_cast<int>((m_start.y+m_size.y+ydist)*mmToLogical), wxRound(SELECTED_RADIUS*mmToLogical));
    }
}

wxPoint RectangleElement::start() const
{
    return m_start;
}

void RectangleElement::setStart(const wxPoint &start)
{
    m_start = start;
}

wxSize RectangleElement::size() const
{
    return m_size;
}

void RectangleElement::setSize(const wxSize &size)
{
    m_size = size;
}

LineStyle RectangleElement::lineStyle() const
{
    return m_style;
}

void RectangleElement::setLineStyle(const LineStyle &style)
{
    m_style = style;
}

wxXmlNode *RectangleElement::createXmlNode() const
{
    wxXmlNode *node = new wxXmlNode(wxXML_ELEMENT_NODE, "Element");
    node->AddAttribute("Type", wxString::Format("%d", RECTANGLE));
    node->AddAttribute("Section", wxString::Format("%d", m_section));
    node->AddAttribute("StartX", wxString::Format("%d", m_start.x));
    node->AddAttribute("StartY", wxString::Format("%d", m_start.y));
    node->AddAttribute("Width", wxString::Format("%d", m_size.x));
    node->AddAttribute("Height", wxString::Format("%d", m_size.y));
    node->AddAttribute("anchor", wxString::Format("%d", m_anchor));
    node->AddAttribute("fillet", wxString::Format("%d", m_fillet));
    node->AddAttribute("Page", wxString::Format("%d", m_page));
    node->AddChild(m_style.createXmlNode());
    return node;
}

bool RectangleElement::fromXmlNode(const wxXmlNode *node)
{
    if(!node) return false;
    if(static_cast<ELEMENTTYPE>(wxAtoi(node->GetAttribute("Type"))) != RECTANGLE) return false;
    m_start = wxPoint(wxAtoi(node->GetAttribute("StartX", "0")), wxAtoi(node->GetAttribute("StartY", "0")));
    m_size = wxSize(wxAtoi(node->GetAttribute("Width", "0")), wxAtoi(node->GetAttribute("Height", "0")));
    m_style.fromXmlNode(node->GetChildren());
    m_page = wxAtoi(node->GetAttribute("Page", "1"));
    m_section = static_cast<DOCSECTION>(wxAtoi(node->GetAttribute("Section", "9")));
    m_anchor = wxAtoi(node->GetAttribute("anchor", "0"));
    m_fillet = wxAtoi(node->GetAttribute("fillet", "0"));
    return true;
}

bool RectangleElement::anchor() const
{
    return m_anchor;
}

void RectangleElement::setAnchor(bool anchor)
{
    m_anchor = anchor;
}

int RectangleElement::fillet() const
{
    return m_fillet;
}

void RectangleElement::setFillet(int fillet)
{
    m_fillet = fillet;
}

int RectangleElement::height(bool /*inpx*/) const
{
    if(m_anchor)
    {
        return 0;
    }
    return m_size.y;
}

int RectangleElement::posY() const
{
    if(m_anchor)
    {
        return 0;
    }
    return m_start.y;
}
