/***************************************************************
 * Name:      table.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "table.h"
#include "utils/dxutils.h"
#include <wx/tokenzr.h>
#include <wx/arrimpl.cpp>

WX_DEFINE_OBJARRAY(dxTableRowsPerPageArray)
WX_DEFINE_OBJARRAY(dxTableCellArray)
WX_DEFINE_OBJARRAY(dxTableRowArray)
WX_DEFINE_OBJARRAY(dxTableArray)
WX_DEFINE_OBJARRAY(dxTableColumnsSplitArray)

TableTitle::TableTitle()
    : m_title(wxEmptyString), m_style(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME)))), m_alignment(wxTEXT_ALIGNMENT_CENTER)
{
}

TableTitle::TableTitle(const wxString &title, wxTextAttrAlignment alignment, TextStyle style)
    : m_title(title), m_style(style), m_alignment(alignment)
{
}

bool TableTitle::isValid() const
{
    return m_title != wxEmptyString;
}

wxString TableTitle::title() const
{
    return m_title;
}

void TableTitle::setTitle(const wxString &title)
{
    m_title = title;
}

TextStyle TableTitle::style() const
{
    return m_style;
}

void TableTitle::setStyle(const TextStyle &style)
{
    m_style = style;
}

wxTextAttrAlignment TableTitle::alignment() const
{
    return m_alignment;
}

void TableTitle::setAlignment(const wxTextAttrAlignment &alignment)
{
    m_alignment = alignment;
}

TableRowsPerPage::TableRowsPerPage(int rowStart, int rowEnd)
    : m_rowStart(rowStart), m_rowEnd(rowEnd)
{
}

int TableRowsPerPage::rowStart() const
{
    return m_rowStart;
}

void TableRowsPerPage::setRowStart(int rowStart)
{
    m_rowStart = rowStart;
}
int TableRowsPerPage::rowEnd() const
{
    return m_rowEnd;
}

void TableRowsPerPage::setRowEnd(int rowEnd)
{
    m_rowEnd = rowEnd;
}


TableCell::TableCell(const wxString &value)
    : m_value(value), m_alignment(wxTEXT_ALIGNMENT_LEFT)
{
}

wxString TableCell::value() const
{
    return m_value;
}

void TableCell::setValue(const wxString &value)
{
    m_value = value;
}

TextStyle TableCell::style() const
{
    return m_style;
}

void TableCell::setStyle(const TextStyle &style)
{
    m_style = style;
}

wxTextAttrAlignment TableCell::alignment() const
{
    return m_alignment;
}

void TableCell::setAlignment(const wxTextAttrAlignment &alignment)
{
    m_alignment = alignment;
}

Table::Table()
    : m_style(TextStyle()), m_columns(-1), m_title(TableTitle(wxEmptyString)), m_headerHeight(0.0)
{
}

Table::Table(int columns)
    : m_style(TextStyle()), m_columns(columns), m_title(TableTitle(wxEmptyString)), m_headerHeight(0.0)
{
    for(int i=0; i<m_columns; i++)
        m_columsWidth.Add(0);
    m_splits.Add(TableColumnsSplit(0,columns-1));
}

bool Table::isValid() const
{
    return m_columns != -1;
}

int Table::columns() const
{
    return m_columns;
}

void Table::setColumns(int columns)
{
    if(m_columns != columns)
    {
        m_title = TableTitle();
        m_header.Clear();
        m_rows.Clear();
        m_columsWidth.Clear();
        m_columns = columns;
        for(int i=0; i<m_columns; i++)
            m_columsWidth.Add(0);
        m_splits.Clear();
        m_splits.Add(TableColumnsSplit(0,columns-1));
    }
}

TableTitle Table::tableTitle() const
{
    return m_title;
}

void Table::setTableTitle(const TableTitle &title)
{
    m_title = title;
}

dxTableCellArray Table::header() const
{
    return m_header;
}

void Table::addCellToHeader(const wxString &value)
{
    TableCell cell(value);
    cell.setAlignment(wxTEXT_ALIGNMENT_LEFT);
    cell.setStyle(m_style);
    m_header.Add(cell);
}

void Table::addRow()
{
    m_rows.Add(dxTableCellArray());
    m_rowsHeight.Add(0.0);
}

void Table::addCell(const wxString &value)
{
    TableCell cell(value);
    cell.setAlignment(wxTEXT_ALIGNMENT_LEFT);
    cell.setStyle(m_style);
    m_rows.Item(m_rows.GetCount()-1).Add(cell);
}

void Table::addCell(const wxString &value, wxTextAttrAlignment alignment)
{
    TableCell cell(value);
    cell.setAlignment(alignment);
    cell.setStyle(m_style);
    m_rows.Item(m_rows.GetCount()-1).Add(cell);
}

void Table::addCell(const wxString &value, wxTextAttrAlignment alignment, const TextStyle &style)
{
    TableCell cell(value);
    cell.setAlignment(alignment);
    cell.setStyle(style);
    m_rows.Item(m_rows.GetCount()-1).Add(cell);
}

dxTableRowArray Table::rows() const
{
    return m_rows;
}

void Table::setColumnWidth(size_t column, int width)
{
    if(column >= static_cast<size_t>(m_columns))
        return;
    m_columsWidth[column] = width;
}

wxArrayDouble Table::columsWidth() const
{
    return m_columsWidth;
}

wxArrayDouble Table::rowsHeight() const
{
    return m_rowsHeight;
}

double Table::headerHeight() const
{
    return m_headerHeight;
}

void Table::refreshCellsSize()
{
    for(size_t i=0; i < static_cast<size_t>(m_columns); i++)
        calculateColumnWidth(i);
    calculateHeaderHeight();
    for(size_t i=0; i < m_rows.GetCount(); i++)
        calculateRowHeight(i);
}

void Table::calculatePages(int marginTop, int marginBottom, int pageHeight)
{
    int height = pageHeight - marginTop - marginBottom;
    int titleHeight = 0;
    if(m_title.isValid())
    {
        titleHeight = wxRound((dxutils::pocetZnaku(m_title.title(), '\n')+1) * 1.2 * m_title.style().font().GetPointSize() * (25.4/72.));
    }
    int rowStart = 0;
    int rowEnd = 0;
    double rows = m_headerHeight;
    for(size_t i=0; i < m_rowsHeight.GetCount(); i++)
    {
        rows += m_rowsHeight[i];
        if((height - (!rowStart?titleHeight:0) - rows) < 0)
        {
            rowEnd = static_cast<int>(i-1);
            m_pages.Add(TableRowsPerPage(rowStart,rowEnd));
            rowStart = static_cast<int>(i);
            rows = m_headerHeight;
        }
    }
    m_pages.Add(TableRowsPerPage(rowStart,static_cast<int>(m_rowsHeight.GetCount()-1)));
}

int Table::pages() const
{
    return static_cast<int>(m_pages.GetCount()*m_splits.GetCount());
}

int Table::pagesWithoutSplit() const
{
    return static_cast<int>(m_pages.GetCount());
}

dxTableRowsPerPageArray Table::rowsPerPage() const
{
    return m_pages;
}

dxTableCellArray Table::cells(size_t index) const
{
    return m_rows.Item(index);
}

int Table::tableWidth() const
{
    int width = 0;
    for(size_t i=0; i<m_columsWidth.GetCount(); i++)
        width += wxRound(m_columsWidth[i]);
    return width;
}

int Table::tableWidthForSplit(size_t index) const
{
    int width = 0;
    for(size_t i=static_cast<size_t>(m_splits[index].columnStart()); i<=static_cast<size_t>(m_splits[index].columnEnd()); i++)
        width += wxRound(m_columsWidth[i]);
    return width;
}

double Table::columnWidth(size_t index) const
{
    return m_columsWidth.Item(index);
}

double Table::rowHeight(size_t index) const
{
    return m_rowsHeight.Item(index);
}

dxTableColumnsSplitArray Table::columnsSplit() const
{
    return m_splits;
}

void Table::setColumnsSplit(dxTableColumnsSplitArray splits)
{
    m_splits = splits;
}

size_t Table::numberOfColumnsSplit() const
{
    return m_splits.GetCount();
}

TextStyle Table::style() const
{
    return m_style;
}

void Table::setStyle(const TextStyle &style)
{
    m_style = style;
}

void Table::calculateColumnWidth(size_t column)
{
    if(column >= static_cast<size_t>(m_columns))
        return;
    if(m_columsWidth[column] != 0.0)
        return;
    double width = 0.0;
    if(m_header.GetCount())
        width = calculateCellWidth(m_header[column]);
    for(size_t i=0; i < m_rows.GetCount(); i++)
    {
        width = wxMax(calculateCellWidth(m_rows[i].Item(column)), width);
    }
    m_columsWidth[column] = width;
}

double Table::calculateCellWidth(TableCell cell)
{
    wxScreenDC dc;
    int iw, ih, iwl;
    wxFont font(cell.style().font());
    wxArrayString lines = wxStringTokenize(cell.value(), wxT("\n"));
    iw = 0;
    for(size_t l = 0; l < lines.GetCount(); ++l)
    {
        dc.GetTextExtent(lines[l], &iwl, &ih, nullptr, nullptr, &font);
        if(iwl>iw) iw = iwl;
    }
    return 25.4 * (iw / static_cast<double>(dc.GetPPI().x)) + 2*CELL_TEXT_OFFSET;
}

int Table::numberOfWrappedLines(dxTableCellArray row)
{
    int lnum = 1;
    wxScreenDC dc;
    for(int c=0; c<m_columns; ++c)
    {
        TableCell cell = row.Item(static_cast<size_t>(c));
        double cw = dxutils::mmToPx(0.9*m_columsWidth[static_cast<size_t>(c)], &dc, true);
        dc.SetFont(cell.style().font());
        int textW, textH, lw;
        wxString wrappedLine, line;
        wxArrayString lines = wxStringTokenize(cell.value(), wxT("\n"));
        lw = static_cast<int>(lines.GetCount());
        for(size_t l = 0; l < lines.GetCount(); ++l)
        {
            line = lines[l];
            wrappedLine = wxT("");
            size_t space = 0;
            for(size_t w = 0; w < line.Len(); ++w)
            {
                wrappedLine += line[w];
                if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
                dc.GetTextExtent(wrappedLine, &textW, &textH);
                if(textW > cw && space)
                {
                    wrappedLine = wxT("");
                    w = space;
                    lw++;
                }
            }
        }
        if(lw > lnum)
            lnum = lw;
    }
    return lnum;
}

void Table::calculateHeaderHeight()
{
    if(!m_header.GetCount())
    {
        m_headerHeight = 0.0;
        return;
    }
    int lines = numberOfWrappedLines(m_header);
    m_headerHeight = 0.0;
    for(size_t i=0; i<m_header.GetCount(); i++)
    {
        m_headerHeight = wxMax(m_headerHeight, lines*calculateCellHeight(m_header[i]));
    }
}

void Table::calculateRowHeight(size_t row)
{
    if(row >= m_rows.GetCount())
        return;
    dxTableCellArray cells = m_rows[row];
    int lines = numberOfWrappedLines(cells);
    double height = 0.0;
    for(size_t i=0; i<cells.GetCount(); i++)
    {
        height = wxMax(height, lines*calculateCellHeight(cells[i]));
    }
    m_rowsHeight[row] = height;
}

double Table::calculateCellHeight(TableCell cell)
{
    return 1.2 * cell.style().font().GetPointSize() * (25.4/72.);
}


TableColumnsSplit::TableColumnsSplit(int columnStart, int columnEnd)
    : m_columnStart(columnStart), m_columnEnd(columnEnd)
{
}

int TableColumnsSplit::columnStart() const
{
    return m_columnStart;
}

void TableColumnsSplit::setColumnStart(int columnStart)
{
    m_columnStart = columnStart;
}

int TableColumnsSplit::columnEnd() const
{
    return m_columnEnd;
}

void TableColumnsSplit::setColumnEnd(int columnEnd)
{
    m_columnEnd = columnEnd;
}
