/***************************************************************
 * Name:      table.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef DXTABLE_H
#define DXTABLE_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/dynarray.h>
#include "styles.h"

class TableTitle
{
public:
    TableTitle();
    TableTitle(const wxString &title, wxTextAttrAlignment alignment=wxTEXT_ALIGNMENT_CENTER,
               TextStyle style=TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));

    bool isValid() const;
    wxString title() const;
    void setTitle(const wxString &title);
    TextStyle style() const;
    void setStyle(const TextStyle &style);
    wxTextAttrAlignment alignment() const;
    void setAlignment(const wxTextAttrAlignment &alignment);
private:
    wxString m_title;
    TextStyle m_style;
    wxTextAttrAlignment m_alignment;
};

class TableRowsPerPage
{
public:
    TableRowsPerPage(int rowStart, int rowEnd);

    int rowStart() const;
    void setRowStart(int rowStart);
    int rowEnd() const;
    void setRowEnd(int rowEnd);
private:
    int m_rowStart;
    int m_rowEnd;
};

WX_DECLARE_OBJARRAY(TableRowsPerPage, dxTableRowsPerPageArray);

class TableColumnsSplit
{
public:
    TableColumnsSplit(int columnStart, int columnEnd);

    int columnStart() const;
    void setColumnStart(int columnStart);
    int columnEnd() const;
    void setColumnEnd(int columnEnd);
private:
    int m_columnStart;
    int m_columnEnd;
};

WX_DECLARE_OBJARRAY(TableColumnsSplit, dxTableColumnsSplitArray);

class TableCell
{
public:
    explicit TableCell(const wxString &value);

    wxString value() const;
    void setValue(const wxString &value);
    TextStyle style() const;
    void setStyle(const TextStyle &style);
    wxTextAttrAlignment alignment() const;
    void setAlignment(const wxTextAttrAlignment &alignment);
private:
    wxString m_value;
    wxTextAttrAlignment m_alignment;
    TextStyle m_style;
};

WX_DECLARE_OBJARRAY(TableCell, dxTableCellArray);
WX_DECLARE_OBJARRAY(dxTableCellArray, dxTableRowArray);

class Table
{
public:
    Table();
    explicit Table(int columns);

    bool isValid() const;
    int columns() const;
    void setColumns(int columns);
    TableTitle tableTitle() const;
    void setTableTitle(const TableTitle &title);
    dxTableCellArray header() const;
    void addCellToHeader(const wxString &value);
    void addRow();
    void addCell(const wxString &value);
    void addCell(const wxString &value, wxTextAttrAlignment alignment);
    void addCell(const wxString &value, wxTextAttrAlignment alignment, const TextStyle &style);
    dxTableRowArray rows() const;
    void setColumnWidth(size_t column, int width);
    wxArrayDouble columsWidth() const;
    wxArrayDouble rowsHeight() const;
    double headerHeight() const;
    void refreshCellsSize();
    void calculatePages(int marginTop, int marginBottom, int pageHeight);
    int pages() const;
    int pagesWithoutSplit() const;
    dxTableRowsPerPageArray rowsPerPage() const;
    dxTableCellArray cells(size_t index) const;
    int tableWidth() const;
    int tableWidthForSplit(size_t index) const;
    double columnWidth(size_t index) const;
    double rowHeight(size_t index) const;
    dxTableColumnsSplitArray columnsSplit() const;
    void setColumnsSplit(dxTableColumnsSplitArray splits);
    size_t numberOfColumnsSplit() const;
    TextStyle style() const;
    void setStyle(const TextStyle &style);
private:
    TextStyle m_style;
    int m_columns;
    TableTitle m_title;
    dxTableCellArray m_header;
    dxTableRowArray m_rows;
    dxTableRowsPerPageArray m_pages;
    dxTableColumnsSplitArray m_splits;
    //width a height v mm
    wxArrayDouble m_columsWidth;
    wxArrayDouble m_rowsHeight;
    double m_headerHeight;

    void calculateColumnWidth(size_t column);
    double calculateCellWidth(TableCell cell);
    int numberOfWrappedLines(dxTableCellArray row);
    void calculateHeaderHeight();
    void calculateRowHeight(size_t row);
    double calculateCellHeight(TableCell cell);
};

WX_DECLARE_OBJARRAY(Table, dxTableArray);

#endif  /* DXTABLE_H */

